import java.io.StreamTokenizer;
import java.io.StringReader;

public class LanguageUnderstanding{

	private Tql tql;
	private Context c;
	private String[] topicAndType;

	private String[] personRelatedTopics =
	{
		"person", "roomfree", "firstname", "lastname", "agent", "man", "woman", "telephone", "address", "title", "department"
	};

	private String[] specificTopics =
	{
		"roomnumber", "marvin", "robot", "tuc", "I", "self", "exit", "entry", "stair", "lift", "toilet", "clock", "day", "date", "time", "quit", "gentlementoilet", "ladiestoilet", "noon"
	};

	private String[] unspecificLocation =
	{
		"location"
	};

	public LanguageUnderstanding(Tql tql, Context c){
		this.tql = tql;
		this.c = c;
	}

	public boolean isPersonRelated(String s){
		for(int i=0; i<personRelatedTopics.length; i++){
			if(s.equals(personRelatedTopics[i])) return true;
		}
		return false;
	}

	public boolean isSpecific(String s){
		for(int i=0; i<specificTopics.length; i++){
			if(s.equals(specificTopics[i])) return true;
		}
		return false;
	}

	public boolean isUnspecificLocation(String s){
		for(int i=0; i<unspecificLocation.length; i++){
			if(s.equals(unspecificLocation[i])) return true;
		}
		return false;
	}

	public String[] getTopicAndType(){
		topicAndType = new String[3];
		topicAndType[1] = MarvinDialog.DEFAULT;
		topicAndType[2] = "";

		//Topic bus
		if(c.bus()){
			topicAndType[0] = "bus";
		}
		//A 'which'-question has been asked.
		else if(tql.questionClass() == MarvinDialog.WHICH){
			classWhich();
		}
		//A 'test'-question has been asked.
		else if(tql.questionClass() == MarvinDialog.TEST){
			classTest();
		}
		//A 'new'-phrase has been uttered
		else if(tql.questionClass() == MarvinDialog.NEW){
			classNew();
		}
		//An 'explain'-phrase has been uttered
		else if(tql.questionClass() == MarvinDialog.EXPLAIN){
			classExplain();
		}
		//A 'do'-phrase has been uttered
		else if(tql.questionClass() == MarvinDialog.DO){
			classDo();
		}

		return topicAndType;
	}

	public String getTopic(){
		String[] temp = getTopicAndType();
		return temp[0];
	}

	private void classWhich(){
		topicAndType[2] += "which";

		String whichContent = tql.getWhichContent();
		String whatIs = tql.whatIs(whichContent);

		normalAnswers(whatIs);
	}

	private void classTest(){
		topicAndType[2] += "test";

		String testType = tql.getFirstRelationType();
		String whatIs;
		if(testType.equals("knowthing")){
			topicAndType[2] += "->knowthing";
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(testType.equals("nrel")){
			topicAndType[2] += "->nrel";
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(testType.equals("know")){
			topicAndType[2] += "->know";
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(testType.equals("be1")){
			topicAndType[2] += "->be1";
			whatIs = tql.whatIs(tql.getUnknown());
			normalAnswers(whatIs);
		}
		else if(testType.equals("show")){
			topicAndType[2] += "->show";
			topicAndType[1] = MarvinDialog.SHOW;

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(testType.equals("accompany")){
			topicAndType[2] += "->accompany";
			topicAndType[1] = MarvinDialog.ACCOMPANY;

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(testType.equals("go")){
			topicAndType[2] += "->go";
			topicAndType[1] = MarvinDialog.GO;

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else{
			topicAndType[2] += "->"+testType;
			whatIs = tql.whatIs(tql.getThirdAtom());
			normalAnswers(whatIs);
		}
	}

	private void classNew(){
		topicAndType[2] += "new";

		String newType = tql.getFirstRelationType();
		String whatIs;
		if(newType.equals("go")){
			topicAndType[2] += "->go";
			topicAndType[1] = MarvinDialog.GO;

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(newType.equals("nrel")||newType.equals("wonder")||newType.equals("believe")||newType.equals("meet")){
			topicAndType[2] += "->"+newType;
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else{
			topicAndType[2] += "->"+newType;
			whatIs = tql.whatIs(tql.getThirdAtom());
			normalAnswers(whatIs);
		}
	}

	private void classExplain(){
		topicAndType[2] += "explain";

		String explainType = tql.getFirstRelationType();
		String whatIs;
		if(explainType.equals("go")){
			topicAndType[2] += "->go";

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(explainType.equals("nrel")){
			topicAndType[2] += "->nrel";
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else{
			topicAndType[2] += "->"+explainType;
			whatIs = tql.whatIs(tql.getThirdAtom());
			normalAnswers(whatIs);
		}
	}

	private void classDo(){
		topicAndType[2] += "do";

		String doType = tql.getFirstRelationType();

		//This if-sentence removes any relation found as first that doesn't provide information about what to 'do'
		if(doType.equals("nrel") || doType.equals("be1") || doType.equals("know")){
			tql.getUnknownAndRemove();
			doType = tql.getFirstRelationType();
		}

		String whatIs;
		if(doType.equals("knowthing")){
			topicAndType[2] += "->knowthing";
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(doType.equals("nrel")){
			topicAndType[2] += "->nrel";
			tql.getUnknownAndRemove();
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(doType.equals("know")){
			topicAndType[2] += "->know";
			tql.getUnknownAndRemove();
			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(doType.equals("be1")){
			topicAndType[2] += "->be1";
			whatIs = tql.whatIs(tql.getUnknown());
			normalAnswers(whatIs);
		}
		else if(doType.equals("show")){
			topicAndType[2] += "->show";
			topicAndType[1] = MarvinDialog.SHOW;

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(doType.equals("accompany")){
			topicAndType[2] += "->accompany";
			topicAndType[1] = MarvinDialog.ACCOMPANY;

			whatIs = tql.whatIs(tql.getUnknownAndRemove());
			normalAnswers(whatIs);
		}
		else if(doType.startsWith("quit(")){
			topicAndType[2] += "->"+doType;
			topicAndType[0] = "quit";
			if(quitContains(doType).equals("youarewelcome")) topicAndType[1] = MarvinDialog.YOUAREWELCOME;
			else if(quitContains(doType).equals("bye")) topicAndType[2] = MarvinDialog.BYE;
		}
		else{
			topicAndType[2] += "->"+doType;
			whatIs = tql.whatIs(tql.getThirdAtom());
			normalAnswers(whatIs);
		}
	}

	private void normalAnswers(String w){
		String whatIs = w;

		if(whatIs == null || whatIs.equals("empty")){
			if(tql.personExists()){
				topicAndType[2] += "->person\n";
				topicAndType[0] = "person";
			}
			else if(tql.roomExists()){
				topicAndType[2] += "->room("+tql.getRoom()+")\n";
				if(tql.getRoom().startsWith("free")) topicAndType[0] = "roomfree";
				else topicAndType[0] = "roomnumber";
			}
			else{
				topicAndType[2] += "->empty/null\n";
				if(topicAndType[1].equals(MarvinDialog.PLACE)||
					topicAndType[1].equals(MarvinDialog.GO)||
					topicAndType[1].equals(MarvinDialog.ROUTEPLAN)||
					topicAndType[1].equals(MarvinDialog.ACCOMPANY)){
					topicAndType[0] = "location";
				}
				else topicAndType[0] = "null";
			}
		}
		else if(whatIs.equals("place")){
			if(topicAndType[1].equals(MarvinDialog.DEFAULT)) topicAndType[1] = MarvinDialog.PLACE;
			topicAndType[2] += "->place";
			whatIs = tql.whatIs(whatIs, whatIs);
			normalAnswers(whatIs);
		}
		else if(whatIs.equals("office")){
			topicAndType[2] += "->office";
			whatIs = tql.whatIs(whatIs, whatIs);
			normalAnswers(whatIs);
		}
		else if(whatIs.equals("floor")){
			topicAndType[2] += "->floor";
			whatIs = tql.whatIs(tql.getInverseIs(whatIs), whatIs);
			normalAnswers(whatIs);
		}
		else if(whatIs.equals("meeting")){
			topicAndType[2] += "->meeting";
			whatIs = tql.whatIs(whatIs, whatIs);
			normalAnswers(whatIs);
		}
		else if(whatIs.equals("routeplan")){
			topicAndType[1] = MarvinDialog.ROUTEPLAN;
			topicAndType[2] += "->routeplan";
			whatIs = tql.whatIs(whatIs, whatIs);
			normalAnswers(whatIs);
		}
		else if(whatIs.equals("street")){
			topicAndType[1] = MarvinDialog.ROUTEPLAN;
			topicAndType[2] += "->street";
			whatIs = tql.whatIs(whatIs, whatIs);
			normalAnswers(whatIs);
		}
		else if(whatIs.equals("room") || whatIs.equals("meetingroom")){
			topicAndType[2] += "->room\n";

			String tempRoomString = tql.getInverseIs(whatIs);
			if(tempRoomString.startsWith("free")) topicAndType[0] = "roomfree";
			else topicAndType[0] = "roomnumber";
		}
		else if(whatIs.equals("telephone")){
			topicAndType[1] = MarvinDialog.TELEPHONE;
			topicAndType[2] += "->telephone\n";
			topicAndType[0] = "telephone";
		}
		else if(whatIs.equals("address")){
			topicAndType[1] = MarvinDialog.ADDRESS;
			topicAndType[2] += "->address\n";
			topicAndType[0] = "address";
		}
		else if(whatIs.equals("title")){
			topicAndType[1] = MarvinDialog.TITLE;
			topicAndType[2] += "->title\n";
			topicAndType[0] = "title";
		}
		else if(whatIs.equals("department")){
			topicAndType[1] = MarvinDialog.DEPARTMENT;
			topicAndType[2] += "->department\n";
			topicAndType[0] = "department";
		}
		else if(isPersonRelated(whatIs) || isSpecific(whatIs)){
			topicAndType[2] += "->"+whatIs+"\n";
			topicAndType[0] = whatIs;
		}
		else{
			if(tql.isA(whatIs) == null){
				topicAndType[2] += "->unknown("+whatIs+")\n";
				topicAndType[0] = "unknown";
			}
			else{
				whatIs = tql.isA(whatIs);
				normalAnswers(whatIs);
			}
		}
	}

	private String quitContains(String s){
		String contains = "";
		StreamTokenizer st = new StreamTokenizer(new StringReader(s));
		st.ordinaryChar('(');
		st.ordinaryChar(')');
		st.ordinaryChar(',');

		try{
			while(st.ttype != '('){
				st.nextToken();
			}
			int parentesCounter = 1;
			while(parentesCounter > 0){
				st.nextToken();
				if(st.sval != null) contains += st.sval;
				else if(st.ttype == StreamTokenizer.TT_NUMBER) contains += (int)st.nval;
				else if(st.ttype == ',') contains += ",";
				else if(st.ttype == '('){
					contains += "(";
					parentesCounter++;
				}
				else if(st.ttype == ')'){
					if(parentesCounter > 1) contains += ")";
					parentesCounter--;
				}
			}
		}
		catch(Exception e){

		}
		return contains;
	}
}