import java.lang.Math;

public class Node{

	private int roomNumber;
	private int xPos;
	private int yPos;
	private Node parent;
	private int fHat;
	private int cost;

	public Node(int number, int x, int y, Node parent, int cost){
		roomNumber = number;
		this.parent = parent;
		xPos = x;
		yPos = y;
		this.cost = cost;

		calcFHat();
	}

	public int calcPathCost(){
		int r = 0;

		if(parent == null) r = 0;
		else{
			r = cost + parent.calcPathCost();
		}
		return r;
	}

	public int calcPathCost(int newCost, Node newParent){
		int r = 0;

		if(newParent == null) r = 0;
		else{
			r = newCost + newParent.calcPathCost();
		}
		return r;
	}

	private void calcFHat(){
		if(parent == null) fHat = 0;
		else{
			fHat = parent.getFHat() + (int) (
					Math.sqrt(
						(xPos-parent.getX())*(xPos-parent.getX())+(yPos-parent.getY())*(yPos-parent.getY())
					)
				);
		}
	}

	public boolean notAncestor(int a){

		boolean recur = true;

		if(roomNumber == a) recur = false;
		else if(parent == null) recur = true;
		else{
			recur = parent.notAncestor(a);
		}
		return recur;
	}

	public void setNewCost(int c){
		cost = c;
	}

	public void setNewParent(Node n){
		parent = n;
	}

	public int getFHat(){
		return fHat;
	}

	public int getNr(){
		return roomNumber;
	}

	public Node getParent(){
		return parent;
	}

	public int getX(){
		return xPos;
	}

	public int getY(){
		return yPos;
	}

	public int getCost(){
		return cost;
	}
}