import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.ArrayList;

public class Tql{

	private ArrayList tqlElements;

	public void parseNewAnswer(String s){
			resetTql();

			StreamTokenizer st = new StreamTokenizer(new StringReader(s));

			st.ordinaryChar(',');
			st.ordinaryChar('*');
			st.ordinaryChar('(');
			st.ordinaryChar(')');
			st.ordinaryChar(' ');
			st.ordinaryChar('/');

			try{
				while(st.ttype != StreamTokenizer.TT_EOF){
					if(st.sval != null && st.sval.equals("TQL")){
						st.nextToken();
						while(st.ttype == '*' || st.ttype == ' '){
							st.nextToken();
						}
						while(st.ttype != '*'){
							String newElementString = "";
							int parentesCounter = 0;
							while((st.ttype != ',' && st.ttype != '*') || parentesCounter > 0){
								if(st.sval != null){
									newElementString += st.sval;
								}
								else{
									if(st.ttype == StreamTokenizer.TT_NUMBER){
										newElementString += (int)st.nval;
									}
									else if(st.ttype == ' '){
										newElementString += " ";
									}
									else if(st.ttype == ','){
										newElementString += ",";
									}
									else if(st.ttype == '('){
										newElementString += "(";
										//inParentes = true;
										parentesCounter ++;
									}
									else if(st.ttype == ')'){
										newElementString += ")";
										//inParentes = false;
										parentesCounter --;
									}
									else if(st.ttype == '/'){
										newElementString += "/";
									}
								}
								st.nextToken();
							}
							tqlElements.add(new TqlElement(newElementString));
							st.nextToken();
						}
					}
					st.nextToken();
				}
			}
			catch(Exception e){
				System.out.println("Exception while performing nextToken() in Context.parseNewAnswer():");
				System.out.println(e.toString());
			}

			//In the end, we remove all the event/real/free(x) elements, since we don't need them to extract the meaning we are after
			for(int i=0; i<tqlElements.size(); i++){
				TqlElement testElement = (TqlElement)tqlElements.get(i);
				ArrayList testList = (ArrayList) testElement.getAtoms();
				if( ((String)(testList.get(0))).equals("event")&&
					((String)(testList.get(1))).equals("real")){
						tqlElements.remove(i);
						i--;
					}
			}
	}

	public String getElementStrings(){
		String returnString = "";
		if(!tqlElements.isEmpty()){
			for(int i=0; i<tqlElements.size(); i++){
				returnString += ((TqlElement)tqlElements.get(i)).getElementString() + "\n";
			}
			return returnString;
		}
		else return "No elements";
	}

	public int questionClass(){
		if(!tqlElements.isEmpty()){
			String firstElementString = ((TqlElement) tqlElements.get(0)).getElementString();
			StreamTokenizer st = new StreamTokenizer(new StringReader(firstElementString));
			try{
				st.nextToken();
			}
			catch(Exception e){}
			if(st.sval.equals("which")) return MarvinDialog.WHICH;
			else if(st.sval.equals("test")) return MarvinDialog.TEST;
			else if(st.sval.equals("new")) return MarvinDialog.NEW;
			else if(st.sval.equals("do")) return MarvinDialog.DO;
			else if(st.sval.equals("explain")) return MarvinDialog.EXPLAIN;
			else return MarvinDialog.DUNNO;
		}
		else{
			return MarvinDialog.NO_QUESTION;
		}
	}

	public void resetTql(){
		tqlElements = new ArrayList();
	}

	//////////////////////////////////////////
	//Methods mainly used by AnswerGenerator//
	//////////////////////////////////////////

	/**
	*	Returns the content of a which(x)-element
	*/
	public String getWhichContent(){
		TqlElement firstElement = (TqlElement) tqlElements.get(0);
		return firstElement.getWhichContent();
	}

	/**
	*	Returns the type of the first relation element. Used to find what type of 'test'-question the user has asked
	*/
	public String getFirstRelationType(){
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isRelation()){
				return testElement.getFirstAtom();
			}
		}
		return null;
	}

	/**
	*	Return what something is, based on the current TQL-expression. This method will only go one depth before it returns what something is.
	*/
	public String whatIs(String s){
		return findRelation(s, "", "", true);
	}

	public String whatIs(String s, String bad){
		return findRelation(s, "", bad, true);
	}

	/**
	*	Returns 'inverse is'. If we have an element "free(1)isa place", and String s == place, the method returns free(1)
	*/
	public String getInverseIs(String s){
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isIs()){
				ArrayList atoms = testElement.getAtoms();
				if( ((String)(atoms.get(2))).equals(s) ){
					return (String)atoms.get(0);
				}
			}
		}
		return null;
	}

	/**
	*	Returns the unknown element in a relation. Also removes the element.
	*/
	public String getUnknownAndRemove(){
		String returnString;
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isRelation()){
				returnString = getUnknown(testElement);
				if(returnString == null) returnString = getLast(testElement);	//If there are no unknowns, take the last atom
				tqlElements.remove(i);
				return returnString;
			}
		}
		return null;
	}

	/**
	*	Returns the unknown element in a relation. Also removes the element.
	*/
	public String getUnknown(){
		String returnString;
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isRelation()){
				returnString = getUnknown(testElement);
				if(returnString == null) returnString = getLast(testElement);				//If there are no unknowns, take the last atom
				return returnString;
			}
		}
		return null;
	}

	/**
	*	Returns the third atom from the first relation element.
	*/
	public String getThirdAtom(){
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isRelation()){
				ArrayList atoms = testElement.getAtoms();
				return (String)atoms.get(2);
			}
		}
		return null;
	}

	/**
	*	Returns a room number, if one exists.
	*/
	public String getRoom(){
		String returnString;
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isIs()){
				ArrayList atoms = testElement.getAtoms();
				if( ((String)atoms.get(2)).equals("room") )	return ((String)atoms.get(0));
			}
		}
		return null;
	}

	/**
	*	Checks if the String s is the first atom in a 'isa'-element and returns the third atom if found
	*/
	public String isA(String s){
		String returnString;
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isIs()){
				ArrayList atoms = testElement.getAtoms();
				if( ((String)atoms.get(0)).equals(s) )	return ((String)atoms.get(2));
			}
		}
		return null;
	}


	/**
	*	Returns true if the TQL-expression contains a person, firstname or lastname
	*/
	public boolean personExists(){
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isIs()){
				ArrayList atoms = testElement.getAtoms();
				String testIs = (String)atoms.get(2);
				if(testIs.equals("person") || testIs.equals("firstname") || testIs.equals("lastname")) return true;
			}
		}
		return false;
	}

	/**
	*	Returns true if the TQL-expression contains a room
	*/
	public boolean roomExists(){
		for(int i=0; i<tqlElements.size(); i++){
			TqlElement testElement = (TqlElement) tqlElements.get(i);
			if(testElement.isIs()){
				ArrayList atoms = testElement.getAtoms();
				String testIs = (String)atoms.get(2);
				if(testIs.equals("room")) return true;
			}
		}
		return false;
	}


	/////////////////////////////////////////
	//Private methods ///////////////////////
	/////////////////////////////////////////

	/**
	*	This method is called recursivly to find what the content of String s actually is.
	*
	*	@param badAnswer 		True if the method should not return String s as an answer.
	*	@param previousAnswer	Is used to avoid going into an infinite loop.
	*/
	private String findRelation(String s, String previousAnswer, String dontAnswer, boolean badAnswer){
		if(s==null) return null;
		else if(!s.startsWith("free(") && !s.startsWith("(") && !Character.isDigit(s.charAt(0)) && !badAnswer){
			if(!s.equals(dontAnswer)) return s;
			else return null;
		}
		else if(s.startsWith("(")){
			return "person";
		}
		else{
			for(int i=0; i<tqlElements.size(); i++){
				TqlElement testElement = (TqlElement) tqlElements.get(i);
				//First check the "isa" elements
				if(testElement.isIs()){
					ArrayList atoms = testElement.getAtoms();
					if( ((String)(atoms.get(0))).equals(s) ){
						if(!((String)atoms.get(2)).equals(dontAnswer)){
							return (String)atoms.get(2);
						}
					}
				}
				//If the content of String s wasn't found in the "isa" elements, check the relation elements
				if(testElement.isRelation()){
					ArrayList atoms = testElement.getAtoms();
					for(int j=0; j<atoms.size(); j++){
						if( ((String)atoms.get(j)).equals(s) ){
							tqlElements.remove(i);
							String nextToFind = getRelationFromElement(atoms, s);
							if(nextToFind != null && nextToFind.equals(previousAnswer)) return "empty";
							else return findRelation(getRelationFromElement(atoms, s), s, dontAnswer, false);
						}
					}
				}
			}
			return null; //No relation found
		}
	}

	/**
	*	This method is called with the atoms from a TQL-element and a String as argument, and should return
	*	whatever thing within the TQL-element that the String is related to.
	*/
	private String getRelationFromElement(ArrayList atoms, String s){
		String relationType = (String)atoms.get(0);
		if(relationType.equals("be1")){
			if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(2);
			else return (String)atoms.get(1);
		}
		else if(relationType.equals("be2")){
			if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(2);
			else if( ((String)atoms.get(3)).equals(s) ) return (String)atoms.get(2);
			else{
				return (String)atoms.get(1);
			}
		}
		else if(relationType.equals("sit")){
			if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(2);
			else return (String)atoms.get(1);
		}
		else if(relationType.equals("live") || relationType.equals("work")){
			if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(2);
			else return (String)atoms.get(1);
		}
		else if(relationType.equals("has")){
			if( ((String)atoms.get(3)).equals(s) ) return (String)atoms.get(4);
			else if( ((String)atoms.get(4)).equals(s) ) return (String)atoms.get(3);
			else if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(3);
			else if( ((String)atoms.get(2)).equals(s) ) return (String)atoms.get(4);
			else return null;
		}
		else if(relationType.equals("srel")){
			if( ((String)atoms.get(3)).equals(s) ) return (String)atoms.get(4);
			else if( ((String)atoms.get(4)).equals(s) ) return (String)atoms.get(3);
			else if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(3);
			else if( ((String)atoms.get(2)).equals(s) ) return (String)atoms.get(4);
			else return null;
		}
		else if(relationType.equals("nrel")){
			if( ((String)atoms.get(4)).equals(s) ) return (String)atoms.get(5);
			else if( ((String)atoms.get(5)).equals(s) ) return (String)atoms.get(4);
			else if( ((String)atoms.get(2)).equals(s) ) return (String)atoms.get(4);
			else if( ((String)atoms.get(3)).equals(s) ) return (String)atoms.get(5);
			else return null;
		}
		else if(relationType.equals("event")){
			if( ((String)atoms.get(1)).equals(s) ) return (String)atoms.get(2);
			else return (String)atoms.get(1);
		}
		else if(relationType.equals("knowthing") || relationType.equals("tell")){
			if( ((String)atoms.get(3)).equals(s) ) return (String)atoms.get(2);
			else return (String)atoms.get(3);
		}
		else return null;
	}

	private String getUnknown(TqlElement testElement){
		ArrayList atoms = testElement.getAtoms();
		for(int i=0; i<atoms.size(); i++){
			if( ((String)atoms.get(i)).startsWith("free(") ) return (String)atoms.get(i);
		}
		return null;
	}

	private String getLast(TqlElement testElement){
		ArrayList atoms = testElement.getAtoms();
		return ((String)atoms.get(atoms.size()-1));
	}
}