import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.ArrayList;

public class TqlElement{

	public static final int CLASS = 0;
	public static final int IS = 1;
	public static final int RELATION = 2;

	String elementString;
	String firstAtom;
	int type;
	ArrayList atoms;

	/**
	*	Constructor that puts all the atoms in the TQL-element into an ArrayList. Also sets what kind of TQL-element this is (class, is, relation),
	*	and sets the first atom to String firstAtom for easier access.
	*/
	public TqlElement(String s){
		this.elementString = s;
		atoms = new ArrayList();
		StreamTokenizer st = new StreamTokenizer(new StringReader(elementString));
		st.ordinaryChar('(');
		st.ordinaryChar(')');
		st.ordinaryChar(',');
		st.ordinaryChar('/');
		st.ordinaryChar(' ');
		st.ordinaryChar(92);

		String newElement ="";
		boolean done = false;

		//The parenteses counter routine is used to be able to separate atoms in Strings like (tore,amble)isa person
		boolean parenteses = false;
		boolean fnutt = false;
		int parentesesCounter = 0;

		try{
			st.nextToken();

			while(!done){
				if((st.ttype == ' ' || st.ttype == '/' || st.ttype == StreamTokenizer.TT_EOF) && !parenteses && !fnutt){
					if(!newElement.equals("")) atoms.add(newElement);
					parenteses = false;
					parentesesCounter = 0;
					newElement = "";
				}
				else if((parenteses && parentesesCounter == 0)){
					if(!newElement.equals("")) atoms.add(newElement);
					if(st.sval != null) newElement = st.sval;
					else newElement = "";
					parenteses = false;
					parentesesCounter = 0;
				}
				else if(st.sval != null) newElement += st.sval;
				else if(st.ttype == StreamTokenizer.TT_NUMBER){
					newElement += (int)st.nval;
				}
				else if(st.ttype == '('){
					newElement += "(";
					parenteses = true;
					parentesesCounter++;
				}
				else if(st.ttype == ')'){
					newElement += ")";
					parentesesCounter--;
				}
				else if(st.ttype == 92){
					newElement += "'";
					if(fnutt) fnutt = false;
					else fnutt = true;
				}
				else if(st.ttype == ',') newElement += ",";

				if(st.ttype == StreamTokenizer.TT_EOF) done = true;
				else st.nextToken();
			}
		}
		catch(Exception e){
			System.out.println("Exception trying st.nextToken() in constructor for TqlElement");
			System.out.println(e.toString());
		}

		if(!atoms.isEmpty()){
			firstAtom = (String)atoms.get(0);
			//Check if this is a class-defining TQL-element
			if(firstAtom.startsWith("which(") || firstAtom.equals("test") || firstAtom.equals("new") || firstAtom.equals("explain") || firstAtom.equals("do")){
				type = CLASS;
			}
			//Check if this is an 'is' TQL-element
			else if(atoms.size() > 1 && ((String)atoms.get(1)).equals("isa")){
				type = IS;
			}
			//If it's not a class defining element, it has to be a relation element
			else{
				type = RELATION;
			}
		}
	}

	public boolean isClass(){
		if(type == CLASS) return true;
		else return false;
	}

	public boolean isIs(){
		if(type == IS) return true;
		else return false;
	}

	public boolean isRelation(){
		if(type == RELATION) return true;
		else return false;
	}

	public String getWhichContent(){
		String contains = "";
		String whichString = (String)atoms.get(0);
		StreamTokenizer st = new StreamTokenizer(new StringReader(whichString));
		st.ordinaryChar('(');
		st.ordinaryChar(')');
		st.ordinaryChar(',');

		try{
			while(st.ttype != '('){
				st.nextToken();
			}
			int parentesCounter = 1;
			while(parentesCounter > 0){
				st.nextToken();
				if(st.sval != null) contains += st.sval;
				else if(st.ttype == StreamTokenizer.TT_NUMBER) contains += (int)st.nval;
				else if(st.ttype == ',') contains += ",";
				else if(st.ttype == '('){
					contains += "(";
					parentesCounter++;
				}
				else if(st.ttype == ')'){
					if(parentesCounter > 1) contains += ")";
					parentesCounter--;
				}
			}
		}
		catch(Exception e){

		}
		return contains;
	}

	public String getFirstAtom(){
		return firstAtom;
	}

	public ArrayList getAtoms(){
		return atoms;
	}

	public String getElementString(){
		return elementString;
	}
}