package ldapconnection;

import javax.naming.*;
import javax.naming.directory.*;
import java.io.*;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Hashtable;
import java.util.Collections;
import java.util.regex.*;
import java.util.StringTokenizer;

/**
 * <p>Title: Tagger</p>
 * <p>Description: The class that checks the words and taggs them to different word groups</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: NTNU</p>
 * @author Tor Livar Flugsrud
 * @version 1.0
 */

public class Tagger {

  private String str_base_ntnu = "dc=NTNU,dc=NO";
  private String str_base_hist = "dc=HIST,dc=NO";
  private String str_base_sintef = "dc=SINTEF,dc=NO";
  private String str_base_other = "dc=ANDRE,dc=NO";
  private DirContext m_ctxNTNU;
  //private DirContext m_ctxHiST;
  //private DirContext m_ctxSINTEF;
  //private DirContext m_ctxOther;

  private int m_iCountSearches;

  private ArrayList m_taggedWords;

  private ArrayList exactNames;
  private ArrayList foundFullNamesGn; // givenname
  private ArrayList foundFullNamesSn; // sn, surname
  private ArrayList depList;

  private int m_iExactCount;

  /***
   * Default constructor. Creates the necessary connections to the LDAP server
   */
  public Tagger() {
    m_taggedWords = new ArrayList();
    try {
      Hashtable env = new Hashtable(11);
      env.put(Context.REFERRAL, "throw");
      env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
      env.put(Context.PROVIDER_URL, "ldap://ldap.ntnu.no:389");
      //env.put(Context.PROVIDER_URL, "ldap://ldap.ntnu.no/o=ntnu,c=no");
      m_ctxNTNU = new InitialDirContext(env);
      /*

      env.put(Context.PROVIDER_URL, "ldap://ldap.ntnu.no/o=hist,c=no");
      m_ctxHiST = new InitialDirContext(env);
      env.put(Context.PROVIDER_URL, "ldap://ldap.ntnu.no/o=sintef,c=no");
      m_ctxSINTEF = new InitialDirContext(env);
      env.put(Context.PROVIDER_URL, "ldap://ldap.ntnu.no/o=andre,c=no");
      m_ctxOther = new InitialDirContext(env);
      */
      m_iCountSearches = 0;

      BufferedReader br = new BufferedReader(new FileReader("DepList.txt"));
      String line;
      depList = new ArrayList();
      while ((line=br.readLine()) != null){
		  //System.out.println("added "+line);
		  depList.add(line);
	  }
	  br.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /***
   * Perform the tagging of the words in the array.
   *
   * @param  words    An array of String objects, the words to be tagged. All words in the array is tagged.
   * @return          A String object containing the XML code that taggs all the words.
   */

  public String strTag(String words[])
  {
	exactNames = new ArrayList();
	foundFullNamesGn = new ArrayList();
	foundFullNamesSn = new ArrayList();

	for (int i=0; i<words.length; i++){
		if (words[i].equals("-") && i>0 && i<(words.length-1)){
			String[] newWords = new String[words.length-2];
			for (int j=0; j<newWords.length; j++){
				if (j<(i-1)) newWords[j] = words[j];
				else if (j==(i-1)) newWords[j] = words[j]+words[j+1]+words[j+2];
				else if (j>=i) newWords[j] = words[j+2];
			}
			words = new String[newWords.length];
			for (int j=0; j<newWords.length; j++){
				words[j] = newWords[j];
				//System.out.println("words"+j+": "+words[j]);
			}
		}
	}


    vFindAttrs(words, "givenname", "firstname");
    vFindAttrs(words, "sn", "lastname");
    //vFindAttrs(words, "cn", "pname");
//    vFindAttrs(words, "title", "title");      // takes to long :(


	vFindDeps(words);

	/*System.out.println("FR:");
			for (Object obj : m_taggedWords){
			System.out.println(obj);
		}*/

	filterTaggedWords();


	//System.out.println("tagged: "+m_taggedWords.size()+" foundFullNamesGn: "+foundFullNamesGn.size()+" foundFullNamesSn: "+foundFullNamesSn.size()+" foundExactWords: "+foundExactWords.size());
	//System.out.println(foundFullNamesSn.get(0));
	filterInitial(words);
	/*System.out.println("ETTER:");
			    for (Object obj : m_taggedWords){
						System.out.println(obj);
	}*/



	//System.out.println("FR:");
	//for (Object obj : m_taggedWords){
	//	System.out.println(obj);
	//}

    Collections.sort(m_taggedWords);
    ListIterator it = null;

    /*System.out.println("ETTER:");
    for (Object obj : m_taggedWords){
			System.out.println(obj);
	}*/

    String strXML = "";

    for (int i = 0; i < words.length; i++)
    {
	  if (i<(words.length-2) && words[i].equals(words[i+1])) continue;
      String strClass = "";
      it = m_taggedWords.listIterator();

	  String word = null;
      while (it.hasNext())
      {
        Tagging t = (Tagging)it.next();
        //System.out.println("har med: "+t);
        if (t.strWord().toUpperCase().equals(words[i].toUpperCase())/* || replaceSpecial(words[i]).equals(t.strWord())*/){
			//System.out.println("er her");
		  word = t.m_strTagged;
          strClass += t.toString();
        }//else System.out.println("fyskam "+t.strWord()+" - "+words[i]);
      }
      if (strClass.length() > 0){
        //String strTag = XMLTagger.strTag("word",words[i]) + '\n';
        String strTag = XMLTagger.strTag("word",word) + '\n';
        strTag += XMLTagger.strTag("class",strClass) + '\n';
        strXML += XMLTagger.strTag("tag",strTag) + '\n';
      }
    }
    try{
      strXML = XMLTagger.strTag("result", strXML,
                                new String[] {"sp"}
                                ,
                                new String[] {"" + m_iCountSearches});
    }catch(Exception e)
    {
      strXML = XMLTagger.strTag("result", strXML);
    }

    strXML = XMLTagger.strMarkXML() + strXML;

    return strXML;
  }

  /*
  * Filters the tagged words, matching any initials in front of tagged surname
  */
  private void filterInitial(String[] origWords){
	  boolean added = false;
	  String prename = "";
	  boolean removePrenameTag = false;
	  for (int i=0; i<origWords.length; i++){
		  String origWord = origWords[i].toUpperCase();
		  for (int j=0; j<m_taggedWords.size(); j++){
			  Tagging testTag = (Tagging)m_taggedWords.get(j);
			  String tagRes = testTag.m_strTagged.toUpperCase();
			  if (i>0){
				  String lastWord = origWords[i-1];
			  	  if (tagRes.equals(origWord) && (lastWord.length()==1 || lastWord.length()==2 && lastWord.charAt(1)=='.')){
					  char testInit = lastWord.toLowerCase().charAt(0);
					  for (int k=0; k<foundFullNamesSn.size(); k++){
						  StringTokenizer st = new StringTokenizer((String)foundFullNamesSn.get(k), " ");
						  int tokens = st.countTokens();
						  int l=1;
						  String initialName = "";
						  while (l<tokens){
							  initialName = st.nextToken();
							  l++;
						  }
						  String lastName = st.nextToken();
						  //System.out.println("found initialname: "+initialName+" testinit: "+testInit);
						  if (!initialName.equals("") && (initialName.toLowerCase().charAt(0) == testInit || exactNames.size()>0) && !added){
							  //foundFullNamesGn.add(""+initialName+" "+tagRes);
							  if (foundFullNamesGn.size()>0){
								  StringTokenizer pst = new StringTokenizer((String)foundFullNamesGn.get(0));
								  prename = pst.nextToken()+" ";
								  removePrenameTag = true;
							  }
							  prename = prename.toLowerCase();
							  //System.out.println("legger til tag."+prename+lastWord+" "+m_taggedWords.size());
							  m_taggedWords.add(new Tagging(new String[]{prename+lastWord}, "firstname", 1, 0));
							  //System.out.println("nysize: "+m_taggedWords.size());
							  added = true;
						  }

					  } //for
				  }//if
			  } //if
		  } //for
	  } //for
	  if (removePrenameTag){
		  //System.out.println("removes..");
		  int rem = -1;
		  for (int i=0; i<m_taggedWords.size(); i++){
			  Tagging testTag = (Tagging)m_taggedWords.get(i);
			  //System.out.println("sammenligner "+testTag.m_strTagged+" og "+prename);
			  if(testTag.m_strTagged.toUpperCase().equals(prename.toUpperCase().trim())) rem = i;
		  }
		  if (rem!=-1) {
			  Tagging removed = (Tagging)m_taggedWords.remove(rem);
			  //System.out.println("removed: "+removed);
		  }
	  }
  }



  /*
  * Filters the tagged words and removes unnecessary results.
  *
  * Uses the list foundFullNames to determine if _one_ full name is found.
  * If it is, all taggings which is not part of that name is removed from the m_taggedWords list.
  */
  private void filterTaggedWords(){
	  if (exactNames.size()==0){
		  //System.out.println("finner eksakt navn.."+foundFullNamesSn.size());
		  for (int i=0; i<foundFullNamesGn.size(); i++){
			  String nameI = (String)foundFullNamesGn.get(i);
			  for (int j=0; j<foundFullNamesSn.size(); j++){
				  String nameJ = (String)foundFullNamesSn.get(j);
				  //System.out.println("sammenligner: "+nameI+" - "+nameJ);
				  if (nameI.equals(nameJ) && !exactNames.contains(nameI)){
					  exactNames.add(nameI);
					  //System.out.println("found exact name: "+nameI);
				  }
			  }
		  }
	  }
	  if (exactNames.size()>0){
		  String mostExactName = null;
		  //for (String n : exactNames){
			 for (int i=0; i<exactNames.size(); i++){
				 String n = (String)exactNames.get(i);
			  if (mostExactName==null || countMatches(n," ")>countMatches(mostExactName," ") ||
			  	(countMatches(n," ")==countMatches(mostExactName," ") && mostExactName.length()<n.length())) mostExactName = n;
		  }
		  mostExactName = mostExactName.toUpperCase();
		  //System.out.println("exactsize: "+exactNames.size()+"; "+mostExactName);
		  for (int i=0; i<m_taggedWords.size(); i++){
			  //System.out.println("tester tag: "+(Tagging)m_taggedWords.get(i));
			  Tagging testTag = (Tagging)m_taggedWords.get(i);
			  String tagRes = testTag.m_strTagged.toUpperCase();
			  //System.out.println(mostGeneralName.indexOf(tagRes)+tagRes+mostGeneralName+"-");
			  //System.out.println(tagRes+" - "+replaceSpecial(tagRes)+" - "+mostExactName);
			  int indexOfTagRes = mostExactName.indexOf(tagRes);
			  if (indexOfTagRes==-1) indexOfTagRes = mostExactName.indexOf(replaceSpecial(tagRes));
			  if ((indexOfTagRes>0 && indexOfTagRes!=(mostExactName.length()-tagRes.length()) && mostExactName.indexOf("-")==-1) ||
			  		(indexOfTagRes==0 && !testTag.m_strClass.equals("firstname")) && !testTag.m_strClass.equals("department") ||
			  		(indexOfTagRes<0 && !testTag.m_strClass.equals("department"))){
				  m_taggedWords.remove(i);
				  //System.out.println("fjernet her "+tagRes);
				  i--;
			  }
		  }
	  }else{
		  // No exact names found. Remove tagged lastname if firstname is tagged exact..
		  //System.out.println("no exact names");
		  for (int i=0; i<m_taggedWords.size(); i++){
			  Tagging testTag = (Tagging)m_taggedWords.get(i);
			  String tagRes = testTag.m_strTagged.toUpperCase();
			  if (testTag.m_strClass.equals("firstname") && testTag.m_iExactCount>0){
			  	  for (int j=0; j<m_taggedWords.size(); j++){
					  Tagging otherTag = (Tagging)m_taggedWords.get(j);
					  if (otherTag.m_strClass.equals("lastname")){
						  //System.out.println("fjernet ikkeeksakt lastname "+otherTag.m_strTagged);
						  m_taggedWords.remove(j);
						  j--;
					  }
				  }
			  }else if (testTag.m_strClass.equals("lastname") && testTag.m_iExactCount>0){
				  for (int j=0; j<m_taggedWords.size(); j++){
					  Tagging otherTag = (Tagging)m_taggedWords.get(j);
					  if (otherTag.m_strClass.equals("firstname")){
						  //System.out.println("fjernet ikkeeksakt firstname");
						  m_taggedWords.remove(j);
						  j--;
					  }
				  }
			  }
		  }
	  }

	  //int remove = -1;
	  //String shortestStr = "";

	  // (second) removes tagged words that are part of a tagging with multiple words (eg. removes lastname 'bjrn', if 'bjrn kre' is tagged as firstname)
	  //          does _not_ remove anything if both taggings are just one word (eg. 'tore' and 'toresen')

	  // (first) removes all but the longest tagged string for each class (eg. removes lastname 'telefax' if 'telefax sintef' is tagged as lastname)
	  ArrayList removeList = new ArrayList();
	  for (int i=0; i<m_taggedWords.size(); i++){
		  Tagging firstTag = (Tagging)m_taggedWords.get(i);
		  for (int j=(i+1); j<m_taggedWords.size(); j++){
			  Tagging secondTag = (Tagging)m_taggedWords.get(j);
			  //System.out.println("sammenligner "+i+firstTag.m_strTagged+" med "+j+secondTag.m_strTagged);
			  if (firstTag.m_strClass.equals(secondTag.m_strClass)){
				  //if (firstTag.m_iExactCount!=secondTag.m_iExactCount){
				  int comp = firstTag.compareTo(secondTag);
				 // System.out.println(""+comp);
				  if (comp < 0 /*firstTag.m_strTagged.length() > secondTag.m_strTagged.length()*/){
					  //m_taggedWords.remove(secondTag);
					  //j--;
					  removeList.add(secondTag);
					  //System.out.println("fjernet lik class: "+secondTag);
				  }else{
					  //m_taggedWords.remove(firstTag);
					  //i--; j--;
					  removeList.add(firstTag);
					  //System.out.println("fjernet lik class: "+firstTag);
				  }
				  //i--;
			  }else if (countMatches(firstTag.m_strTagged, " ")!=countMatches(secondTag.m_strTagged, " ") &&
			  			(firstTag.m_strTagged.indexOf(secondTag.m_strTagged)!=-1 || secondTag.m_strTagged.indexOf(firstTag.m_strTagged)!=-1)){
				  if (firstTag.m_strTagged.indexOf(secondTag.m_strTagged)!=-1){
					  //m_taggedWords.remove(secondTag);
					  //j--;
					  removeList.add(secondTag);
					  //System.out.println("fjernet lik word: "+secondTag);
				  }else{
					  //m_taggedWords.remove(firstTag);
					  //i--; j--;
					  removeList.add(firstTag);
					  //System.out.println("fjernet lik word: "+firstTag);
				  }

			  }
		  }
	  }
	  for (int i=0; i<removeList.size(); i++){
		  m_taggedWords.remove((Tagging)removeList.get(i));
	  }

  }



  /**
   * Checks if a string can be found in the LDAP server as a certain concept
   *
   * This private function does the tagging of each word, or each composition of
   * words. It will be called for every word for every word concept.
   *
   * @param   astr         an array of the words to be tagged (if more than one, treated as a composition of words).
   * @param   strAttrib    the attribute in the LDAP server to search for.
   * @param   strConcept   the TUC concept that the tagg should give.
   * @return               if the word(s) are found as the given attribute, a {@link Tagger.Tagging} object, else null
   */

  private Tagging isAttrib(String[] astr, String strAttrib, String strConcept) {
	String srchStr = "";
	boolean tooShort = false;
	for (int i=0; i<astr.length; i++){
		srchStr += astr[i]+" ";
		if (astr[i]!=null && astr[i].length()<3 && astr.length>1) tooShort = true; // ignore if name has more than one word, and one of them is less than 3 letters
	}
	//System.out.println("*** SKESTRENG: "+srchStr);
	if (tooShort) return null;
    if (astr.length == 1 && astr[0].length() == 1) {
		//System.out.println(astr[0]+" er for kort :'(");
      return null;
    }
    try {
      String strFilter = strMakeCompFilter(astr, strAttrib);
      int i;
      if ( (i = iExists(strFilter)) > 0) {
		  String out ="";
		  for (int j=0; j<astr.length; j++){
			  out+=astr[j];
		  }
//System.out.println("returnerer...");
        return new Tagging(astr, strConcept, i, m_iExactCount);
      }

      String strFiltern;
      // Try capitalizing norwegian characters [] in beginning of words
      strFiltern = strFilter.replaceAll("\\b", "");
      strFiltern = strFiltern.replaceAll("\\b", "");
      strFiltern = strFiltern.replaceAll("\\b", "");
      strFiltern = strFiltern.replaceAll("\\B", "");
	  strFiltern = strFiltern.replaceAll("\\B", "");
      strFiltern = strFiltern.replaceAll("\\B", "");

	  if (strFiltern.indexOf("")!=-1 || strFiltern.indexOf("")!=-1 || strFiltern.indexOf("")!=-1 ||
	  		strFiltern.indexOf("")!=-1 || strFiltern.indexOf("")!=-1 || strFiltern.indexOf("")!=-1 ){
		  //System.out.println("fortsetter..norsk");
		  if ( (i = iExists(strFiltern)) > 0) {
			Tagging t = new Tagging(astr, strConcept, i, m_iExactCount);
			// t.capitalizeNorw(); // skaper problemer
			return t;
		  }
	  }
	  //System.out.println("er her..");

	  String strFilteru = strFiltern;
	  //System.out.println("filter fr: "+strFilteru);
	  strFilteru = replaceSpecial(strFilteru);
      //System.out.println("filter etter: "+strFilteru);
      if (!strFilteru.equals(strFiltern)){
		  //System.out.println("fortsetter..yy");
		  if ( (i = iExists(strFilteru)) > 0) {
			/*String tagged = "";
			for (int j=0; j<astr.length; j++){
				tagged+=replaceSpecial(astr[j]);
			}*/
			//t.setStrTagged(replaceSpecial(tagged));
			Tagging t = new Tagging(astr, strConcept, i, m_iExactCount);

			//System.out.println("ny tagg: "+t);
			return t;
		  }
	  }//else System.out.println("ventle.");

    }
    catch (Exception e) {
		//System.out.println(e);
      // nothing to do - just return null on error
    }
    return null;
  }

  private String replaceSpecial(String input){
	  String output = input;
	  String[][] rChars = { {"", "", "y", "", "", "Y"}, {"", "", "", "", "", ""} };
	  for (int k=0; k<rChars[0].length; k++){
		  output = output.replaceAll(rChars[0][k], rChars[1][k]);
		  if ((k+3)<rChars[1].length) output = output.replaceAll("\\b"+rChars[1][k], rChars[1][k+3]); // capitalize first character
	  }
	  //System.out.println("output: "+output);
	  return output;
  }


  /**
   * Gets a filter and checks if this exists in the LDAP server.
   *
   * Counts the number of times the filter is found, and returns the number.
   * The function also counts the number of exact matches and stores this number
   * in the class member m_iExactCount.
   * <p>
   * <b>Note:</b> The returned number is not necessary the absolute number of items that matches,
   *       since the function won't search any more contexes after a match is found
   * <p>
   * The number of matches, and number of exact matches, is used to give a resonably correct
   * sorting of the tags, so "jens" is marked as
   * a firstname before it is is marked as a lastname (because "jensen" is a lastname).
   *
   * @param strFilter  a String object containing the filter to search for
   * @return           the number of items found
   * @throws Exception If something goes wrong (typically in regex processing)
   *                   can be ignored (treated as if zero occurences is found)
   */
  private int iExists(String strFilter) throws Exception {
	  //System.out.println("filter: "+strFilter);
    int iRet = 0;

    Pattern p = Pattern.compile("(\\w+)=([\\-\\w]+[\\w]+)\\*");
    Matcher m = p.matcher(strFilter);

    String strMatch = "";
    String strAtrId = "";
    boolean boMatchExact = false;
    //if (m.find()){
	int numWords = 0;
	while(m.find()){
      strAtrId = m.group(1);
      if (!strMatch.equals("")) strMatch+=" ";
      strMatch += m.group(2);
      boMatchExact = true;
      numWords++;
      //System.out.println("boexact: "+strMatch+" - "+strAtrId);
    }

    //System.out.println("laget pattern med strAtrId: "+strAtrId+", strMatch: "+strMatch+", strFilter: "+strFilter);

    p = Pattern.compile("\\b" + strMatch + "\\b", Pattern.CASE_INSENSITIVE);

    boolean boSkipCount = false;
    if (strAtrId.equals("title"))
      boSkipCount = true;

    m_iExactCount = 0;

    // Specify the ids of the attributes to return
    String[] attrIDs = {
		//"givenname", "sn" };
        strAtrId.length() > 0 ? strAtrId: "sn"};
    //System.out.println("strAtrId: "+strAtrId);
    if (strAtrId.equals("givenname") || strAtrId.equals("sn")) attrIDs = new String[]{ "givenname", "sn" };
    SearchControls ctls = new SearchControls();
    ctls.setReturningAttributes(attrIDs);
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);

    //System.out.println("sker...");

    // Search subtree for objects using filter


    //System.out.println("***************** "+strFilter);

	iRet += searchForMatch(str_base_ntnu, strFilter, ctls, boSkipCount, boMatchExact, p, m, strAtrId);
	/*if (iRet==0)*/ iRet += searchForMatch(str_base_hist, strFilter, ctls, boSkipCount, boMatchExact, p, m, strAtrId);
	/*if (iRet==0)*/ iRet += searchForMatch(str_base_sintef, strFilter, ctls, boSkipCount, boMatchExact, p, m, strAtrId);
	/*if (iRet==0)*/ iRet += searchForMatch(str_base_other, strFilter, ctls, boSkipCount, boMatchExact, p, m, strAtrId);

    /*if (iRet == 0) {
		System.out.println("sker HIST...");
      //answer = m_ctxHiST.search("", strFilter, ctls);
      answer = m_ctxNTNU.search(str_base_hist, strFilter, ctls);	// endret Tore Bruland
      while (answer.hasMore()) {
		  System.out.println("fant noe p hist...");
        answer.next();
        iRet++;
      }
      answer.close();
      m_iCountSearches++;
    }
    if (iRet == 0) {
      //answer = m_ctxSINTEF.search("", strFilter, ctls);
      answer = m_ctxNTNU.search(str_base_sintef, strFilter, ctls);	// endret Tore Bruland
      while (answer.hasMore()) {
        answer.next();
        iRet++;
      }
      answer.close();
      m_iCountSearches++;
    }
    if (iRet == 0) {
      //answer = m_ctxOther.search("", strFilter, ctls);
      answer = m_ctxNTNU.search(str_base_other, strFilter, ctls);	// endret Tore Bruland
      while (answer.hasMore()) {
        answer.next();
        iRet++;
      }
      answer.close();
      m_iCountSearches++;
    }*/
    if (iRet==1 && numWords>1){
		StringTokenizer st = new StringTokenizer(strMatch, " ");
		boolean firstNameExact = false;
		boolean lastNameExact = false;
		String[] wordsTab = new String[st.countTokens()];
		int i = 0;
		while (st.hasMoreTokens()){
			wordsTab[i] = st.nextToken();
			i++;
		}
		//for (Tagging t : m_taggedWords){
		for (int j=0; j<m_taggedWords.size(); j++){
			Tagging t = (Tagging)m_taggedWords.get(j);
			if (t.m_strClass.equals("firstname") && t.m_strTagged.equals(wordsTab[0]) && t.m_iExactCount > 0) firstNameExact = true;
			else if (t.m_strClass.equals("lastname") && t.m_strTagged.equals(wordsTab[1]) && t.m_iExactCount > 0) lastNameExact = true;
		}
		if (firstNameExact || lastNameExact) m_iExactCount ++;
	}
	//System.out.println("return: "+iRet+" for "+strFilter);

    return iRet; // return  the number of found records
  }

  private int searchForMatch(String str_base, String strFilter, SearchControls ctls,
  				boolean boSkipCount, boolean boMatchExact, Pattern p, Matcher m, String strAtrId){
					//System.out.println("skebase: "+str_base+" strFilter: "+strFilter+" ctls: "+ctls);
	int iRet = 0;
	//boolean output = false;
	try{
		NamingEnumeration answer = m_ctxNTNU.search(str_base, strFilter, ctls);	// endret Tore Bruland
		while (answer.hasMore()) {
			//System.out.println("ny linje i svar");
			if (boSkipCount) break;
			SearchResult sr = (SearchResult) answer.next();
			//if (output) System.out.println("res: "+sr);
			if (boMatchExact){
				//System.out.println("size: "+sr.getAttributes().size());
				NamingEnumeration ne = sr.getAttributes().getAll();
				String fullNameFound = "";
				while(ne.hasMore()){
					Attribute at = (Attribute)ne.next();
					fullNameFound += at.get()+" ";
					//System.out.println("onestep: "+fullNameFound+" - attribute: "+at.getID()+" vals: "+at.size());
				}
				fullNameFound = fullNameFound.trim();
				Attribute at = sr.getAttributes().get(strAtrId);
				String str = (String)at.get();
				if (str.indexOf("..")==0) str = str.substring(2,str.length()); // "..Instituttkontor IDI" ++
				m = p.matcher(str);
				//System.out.println("tester attr: "+str+", m: "+m);
				//if (m.find()){
				if (m.matches()){
				  //System.out.println("fant eksakt: "+str+" med pattern: "+p+" - "+strFilter+" strAtrId: "+strAtrId);
				  m_iExactCount++;
				}//else System.out.println("ikke eksakt");
				//System.out.println("str: "+str+" ("+strAtrId+")"+sr.getAttributes().size());
				//System.out.println("full name found: "+fullNameFound+", strAtrId: "+strAtrId);
				if (strAtrId.equals("givenname")) foundFullNamesGn.add(fullNameFound);
				else if (strAtrId.equals("sn")) foundFullNamesSn.add(fullNameFound);

				int numSrchWords = countMatches(strFilter,"|");
				int numNameWords = countMatches(fullNameFound," ")+1;
				//System.out.println("tester "+fullNameFound+": "+numSrchWords+" _ "+numNameWords);
				if (numSrchWords>1 && numSrchWords == numNameWords) exactNames.add(fullNameFound); // fix for ex Telefax SINTEF Kjemi, where everything is a lastname
			}//else System.out.println("woot");
			iRet++;
		}
		answer.close();
	}catch(Exception e){}
	m_iCountSearches++;
	return iRet;
  }


  private int countMatches(String string, String character){
	int num = 0;
	int index = 0;
	do{
		index = string.indexOf(character, index); // counts the number of words in the search string
		if (index!=-1){
			index++;
			num++;
		}
	}while(index!=-1);
	return num;
  }

  /**
   * Checs if a specified filter gives any returns.
   *
   * @param   strFilter   the filter to check for
   * @return              true if any records coresponding to the filter was found, false if none where found.
   * @throws  Exception   if anything wrong happens with the LDAP connection.
   */
  public boolean boExists(String strFilter) throws Exception {
    // Specify the ids of the attributes to return
    String[] attrIDs = {
        "sn"};
    SearchControls ctls = new SearchControls();
    ctls.setReturningAttributes(attrIDs);
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);

    // Search subtree for objects using filter
	//System.out.println("Sker NTNU: " + strFilter);
    NamingEnumeration answer = m_ctxNTNU.search(str_base_ntnu, strFilter, ctls); // endret Tore Bruland
    m_iCountSearches++;

    boolean boRet = answer.hasMore();
    answer.close();
    if (!boRet) {
	  //System.out.println("Sker HIST: " + strFilter);
      //answer = m_ctxHiST.search("", strFilter, ctls);
      answer = m_ctxNTNU.search(str_base_hist, strFilter, ctls);	// endret Tore Bruland
      boRet = answer.hasMore();
      answer.close();
      m_iCountSearches++;
    }
    if (!boRet) {
	  //System.out.println("Sker SINTEF: " + strFilter);
      //answer = m_ctxSINTEF.search("", strFilter, ctls);
      answer = m_ctxNTNU.search(str_base_sintef, strFilter, ctls); // endret Tore Bruland
      boRet = answer.hasMore();
      answer.close();
      m_iCountSearches++;
    }
    if (!boRet) {
	  //System.out.println("Sker ANDRE: " + strFilter);
      //answer = m_ctxOther.search("", strFilter, ctls);
      answer = m_ctxNTNU.search(str_base_other, strFilter, ctls);	// endret Tore Bruland
      boRet = answer.hasMore();
      answer.close();
      m_iCountSearches++;
    }

    return boRet; // return true if anything is found
  }

  /**
   * Makes a LDAP search filter from a argument name and a list of values to look for
   *
   * @param astrVals    an array of String objects containing the values to look for
   * @param strArg      the argument to search in
   * @return            the filter made of the arguments
   */
  public static String strMakeCompFilter(String[] astrVals, String strArg) {
    if (astrVals.length < 1) {
      return "";
    }

    String strRet = "";
    String strNew = "";
    for (int i = 0; i < astrVals.length - 1; i++) {
      strRet += "(&";
    }
    strRet += "(|(" + strArg + "=" + astrVals[0] + "*)(" + strArg + "=* " +
        astrVals[0] + "*)(" + strArg + "=.." + astrVals[0] + "*)(" + strArg + "=*-" + astrVals[0] + ")(" + strArg + "=" + astrVals[0] + "-*))";
    /*strRet += "(|(" + strArg + "=" + astrVals[0] + "*)(" + strArg + "=* " +
        astrVals[0] + "*))";*/
    for (int i = 1; i < astrVals.length; i++) {
      strRet += "(|(" + strArg + "=" + astrVals[i] + "*)(" + strArg + "=* " +
          astrVals[i] + "*)(" + strArg + "=*-" + astrVals[i] + ")(" + strArg + "=" + astrVals[i] + "-*)))";

    }
    if (!strArg.equals("givenname")) {
      //strRet = "(&" + strRet + "(!(givenname=)))";
      strRet = "(&" + strRet + ")"; // endret Tore Bruland
    }

    return strRet;
  }

  public static String strMakeMailFilter(String[] astrVals, String strArg) {
      if (astrVals.length < 1) {
        return "";
      }

      String strRet = "";
      String strNew = "";
      for (int i = 0; i < astrVals.length - 1; i++) {
        strRet += "(&";
      }
      strRet += "(|(" + strArg + "=*@" + astrVals[0] + ".*)(" + strArg + "=*@*." +astrVals[0] + ".*))";
      /*strRet += "(|(" + strArg + "=" + astrVals[0] + "*)(" + strArg + "=* " +
          astrVals[0] + "*))";*/
      for (int i = 1; i < astrVals.length; i++) {
        strRet += "(|(" + strArg + "=*@" + astrVals[i] + ".*)(" + strArg + "=*@*." +astrVals[i] + ".*))";

      }
      if (!strArg.equals("givenname")) {
        //strRet = "(&" + strRet + "(!(givenname=)))";
        strRet = "(&" + strRet + ")"; // endret Tore Bruland
      }

      return strRet;
  }

  /**
   * Searches in the LDAP server looking for one or more records where the words
   * given are present in the attribute given.
   *
   * Will try to match one ore more successive words from the word list.
   * The results are stored in the member list m_taggedWords
   *
   * @param   astrWords     an array of String objects to look for -
   *                        all must be present in the attribute to generate a match
   * @param   strAtt        the attribute to find the values in
   * @param   strConcept    the concept of the word - is used to tagg the words if
   *                        a match is found
   */
  private void vFindAttrs(String astrWords[], String strAtt, String strConcept) {
	  //System.out.println("-----vFindAttrs kalles-----");
    //Tagging tagOld;
    Tagging tagNew;
    //Hashtable<String, Boolean> foundExact = new Hashtable<String, Boolean>(3);
    boolean foundExact = false;
    //System.out.println("sker i class: "+strConcept);
    //foundExact.put("hei", true);
    //System.out.println("TEST: "+foundExact.get("hei"));
    for (int mainCount = 0; mainCount < astrWords.length; mainCount++) {
		int i=mainCount;
		//System.out.println("** sker p: "+astrWords[i]+", strAtt: "+strAtt+", strConcept: "+strConcept);
      if (LDAPSearcher.strStopwords.indexOf(" " + astrWords[i] + " ") != -1 || astrWords[i].length() <= 2 /*||
          (astrWords[i].length() == 1 && strAtt != "cn") */) {
			  //System.out.println("CONTINUE!!");
        continue;
      }
      //System.out.println("kjrer isAttrib p: "+astrWords[i]);
      if ( (tagNew = isAttrib(new String[] {astrWords[i].toLowerCase()}, strAtt, strConcept)) != null) {
		  String str = astrWords[i];
		  do{
			  //str = astrWords[i];
		  //System.out.println("gr gjennom.."+tagNew);
        //String str = astrWords[i];
        //int j = i;

        //tagOld = tagNew;

        /*while (++i <= astrWords.length) {
          String[] astr = new String[i - j + 1];
          if (i != astrWords.length) {
            for (int k = 0; k <= i - j; k++) {
              astr[k] = astrWords[j + k];
				//System.out.println(k+"legger til i sjekk: "+astr[k]);
            }
          }*/
         //while(i<astrWords.length){
          //tagOld = tagNew;
          //tagNew = isAttrib(astr, strAtt, strConcept);
          //System.out.println("isattrib: ( "+astr.length+", "+strAtt+", "+strConcept+") "+isAttrib(astr, strAtt, strConcept));
          //if (/*i == astrWords.length || (tagNew = isAttrib(astr, strAtt, strConcept)) == null ||*/ true) {
           // if (/*str.length() > 1*/true) { // don't tag one letter word unless initial
			 	//System.out.println("prver  legge til "+str);
			 	boolean okAdd = true;
			 	//if (tagOld.m_iExactCount>0 && strConcept.equals("pname")) m_taggedWords.clear(); // exact full name, clear rest
			 	//if (tagOld.m_iExactCount==0 && strConcept.equals("pname")) okAdd = false; // dont add full name unless its "exact"
				//System.out.println("tagold exact hit: "+tagOld.m_iExactCount);
				//System.out.println("tester  legge til: "+tagNew);
				for (int l=0; l<m_taggedWords.size(); l++){
					Tagging t = (Tagging)m_taggedWords.get(l);
					if (tagNew.m_iExactCount>0){
						if (t.m_iExactCount==0 && (t.m_strClass.equals(strConcept) || t.m_strTagged.equals(str)) ){ // klassen er ikke tagget eksakt fra fr, gjr det n
							//System.out.println("eksaktclass, fjernet: "+t);
							m_taggedWords.remove(l);
							l--;
						}//else if(t.m_iExactCount>0 && t.m_strClass.equals(strConcept)){ okAdd = false; System.out.println("her"); }
					}else if (t.m_iExactCount>0 && t.m_strTagged.equals(str)) okAdd = false; // ordet er tagget eksakt fra fr
					else if(t.m_iExactCount>0 && t.m_strClass.equals(strConcept) && t.m_strTagged.length()>str.length()) okAdd = false; // klassen er tagget eksakt fra fr
				}
				//System.out.println("okadd: "+okAdd);

				if (okAdd && !m_taggedWords.contains(tagNew)){
					/*for (Tagging t : m_taggedWords){
						System.out.println(t+" equals "+tagOld+"? "+t.equals(tagOld));
					}*/
					m_taggedWords.add(tagNew);
              		//System.out.println("la til: "+tagNew+" - "+tagNew.m_iExactCount+"size: "+m_taggedWords.size());
				}
            //}
            //i--; // since the last checked word was not part of a name
            /*if (i<astrWords.length)*/ //str += " " + astrWords[i];
            //System.out.println("ny str: "+str);
            //break;
          //}
          //else {
			  //str += " " + astrWords[i];

			  i++;
			  if (i<astrWords.length){
				  //int tabSize = i-mainCount+1;
				  /*for (int j=0; j<=(i-mainCount); j++){
					  if (astrWords[mainCount+j].equals("-")) tabSize = tabSize-2;
				  }*/

				  String[] astr = new String[i-mainCount+1];
				  //System.out.println("i-mainCount: "+(i-mainCount)+", i: "+i);
				  for (int j=0; j<=(i-mainCount); j++){
					/*if (astrWords[mainCount+j].equals("-") && j>0 && j<(i-mainCount)){
						System.out.println("fant jeg fant");
						astr[j-1]+="-"+astrWords[mainCount+j+1].toLowerCase();
						tabSize = tabSize-2;
						j++;
					}else{*/
						//System.out.println("men:"+astrWords[mainCount+j]+".");
						astr[j] = astrWords[mainCount+j].toLowerCase();
						//str += astr[j]+" ";
						//System.out.println("la til i test: "+astr[j]);
					//}
				  }
				  //str = str.trim();
				  str = "";
				  //String[] astrM = new String[tabSize];
				  for (int j=0; j<astr.length; j++){
					  //astrM[j] = astr[j];
					  str+=astr[j]+" ";
				  }
				 // System.out.println("***** Sker etter streng: "+str.trim());


				  tagNew = isAttrib(astr, strAtt, strConcept);
				  //System.out.println("ferdig str: -"+str+"- res: "+tagNew);
			  }//else System.out.println("i>=astrWords.length"+astrWords.length);
          }while(i<astrWords.length && tagNew != null);
        //} // while
      } // if
    } // for
  } //vFindAttrs

  /**
   * Searches for the given words in the file with the department list.
   *
   * The function looks for words, or combination of successive words, in the
   * department list. Successful taggings are stored in the m_taggedWords member
   * list
   *
   * @param  astrWords   the list of words to find in the department list
   */
  void vFindDeps(String astrWords[]) {
    Tagging taggNew;
    Tagging taggOld;
    for (int i = 0; i < astrWords.length; i++) {
      if (LDAPSearcher.strStopwords.indexOf(" " + astrWords[i] + " ") != -1 ||
          astrWords[i].length() < 3) {
			  //System.out.println("contine..");
        continue;
      }
      //System.out.println("isDepartment(astrWords[i]): "+astrWords[i]+" : "+isDepartment(astrWords[i]));
      taggNew = isDepartment(astrWords[i]);
      //System.out.println(""+astrWords[i]+": "+taggNew);
      if ( taggNew != null) {
	  //while (taggNew != null){
        /*String str = astrWords[i];
        int j = i;

        taggOld = taggNew;

        while (++i <= astrWords.length) {
		  String srchStr = "";
          String[] astr = new String[i - j + 1];
          if (i != astrWords.length) {
            for (int k = 0; k <= i - j; k++) {
              astr[k] = astrWords[j + k];
              srchStr += astr[k]+" ";
            }
            srchStr = srchStr.trim();
            taggOld = taggNew;
          }
          System.out.println("sker p dep: "+srchStr);
          if (i == astrWords.length || (taggNew = isDepartment(astr)) == null) {
            String[] astrKey = new String[astr.length - 1];
            System.arraycopy(astr, 0, astrKey, 0, astr.length - 1);*/
            //System.out.println("fant dep: "+taggNew);
            Tagging oldDepTag = null;
            for (int j=0; j<m_taggedWords.size(); j++){
				Tagging t = (Tagging)m_taggedWords.get(j);
				if (t.m_strClass.equals("department") ) oldDepTag = t;
			}
			boolean okAdd = false;
			if (oldDepTag==null){
				//System.out.println("legger til "+taggNew);
				okAdd = true;
			}else if (oldDepTag!=null && oldDepTag.m_strTagged.length()<taggNew.m_strTagged.length()){
				//System.out.println("fjerner "+oldDepTag+" og legger til "+taggNew);
				m_taggedWords.remove(oldDepTag);
				okAdd = true;
			}//else System.out.println("har funnet bedre tagg fra fr enn "+taggNew);
			if (okAdd){
				m_taggedWords.add(taggNew);
				for (int j=0; j<m_taggedWords.size(); j++){
					Tagging t = (Tagging)m_taggedWords.get(j);
					if (t.m_strTagged.equals(taggNew.m_strTagged) && !t.m_strClass.equals("department") /*&& exactNames.size()==0*/){
						//System.out.println("removed: "+j);
						m_taggedWords.remove(j); // removes other taggins of the word if it is found as department.
						j--;
					}
				}
			}

            //m_taggedWords.add(taggNew); //taggOld
 /*           i--; // since the last checked word was not part of a name
            break;
          }
          else {
            str += " " + astrWords[i];
          }
        }*/
      } //if
    }//for



  }


  /**
   * Creates a regex to use in a grep search in the deps.txt file from a list of words
   *
   * @param   astr   an array of String objects, the words to include in the regex.
   */
  public static String strGrepComb(String[] astr) {
    int iCnt = astr.length;
    String aastr[][] = Permutater.permutate(astr);
    String strRet = "";

    String strBegOfWord;
    String strOs = System.getProperty("os.name");
    if (false /*strOs.startsWith("Win")*/) {
      strBegOfWord = "";
    }
    else {
      strBegOfWord = "\\<";

    }

    for (int i = 0; i < aastr.length; i++) {
      for (int j = 0; j < aastr[i].length; j++) {
        if (i == 0 && j == 0) {
          strRet = strBegOfWord + aastr[i][j];
        }
        else if (j == 0) {
          strRet += "\\|" + strBegOfWord + aastr[i][j];
        }
        else {
          strRet += ".*" + strBegOfWord + aastr[i][j];
        }
      }
    }
    return strRet;
  }

  /**
   * Search for the given string in the department list.
   *
   * @param str  the word to search for
   * @return     if the word was found, a corresponding {@link Tagger.Tagging} object,
   *             if none found records, the function returns <B>null<(B>.
   */
  private Tagging isDepartment(String str) {
	  //System.out.println("isdepartment? "+str);

    try {
		// sjekk fil med vanlige forkortelser, FR db-sk.
	  int i=0;
	  boolean found = false;
	  String depFound = "";
	  while (i<depList.size() && !found){
		  String dep = (String)depList.get(i);
		  //System.out.println("sjekker dep: "+dep);

		  Pattern p = Pattern.compile("\\b" + str + "\\s", Pattern.CASE_INSENSITIVE);
		  Matcher m = p.matcher(dep);
		  if (m.find()){
			  //System.out.println("fant dep: "+str);
			  found = true;
			  depFound = dep;
		  }


		  /*int index = dep.indexOf(str);
		  if (index!=-1 && (index==0 || dep.charAt(index-1)==' ')){
			  found = true;
			  depFound = dep;
		  }*/
		  i++;
	  }
	  //System.out.println("found "+found);
	  if (found) return new Tagging(str, "department", 0, 0);






      //String strFilter = "(|(mail=*@" + str + ".*)(mail=*@*." + str +
      //    ".*))"; //((|(ou= " + str + "*)(ou=* " + str + "*)) __ )
      /*String strFilter = "(|(ou=" + str + "*)(ou=* " + str + "*))";
      int iCount;
      if (boExists(strFilter)) {   /// tar lang tid..
        return new Tagging(str, "department", 0, 0);
      }*/
      /*System.out.println("finner p annet tull..");

      Runtime rt = Runtime.getRuntime();

      String strBegOfWord = null;
      String strOs = System.getProperty("os.name");
      if (strOs.startsWith("Win")) {
        strBegOfWord = "";
      }
      else {
        strBegOfWord = "\\<";

      }
      String strGrep[] = new String[] {
          "grep", "-i", *//* "-c",*/ /*"-q",*/
          /*strBegOfWord +
          str, *//*System.getProperty("user.dir") + System.getProperty("file.separator")+*/
   /*       "deps.txt"};

      String an[] = null;

      Process pGrep = rt.exec(strGrep, null,
                              new File(System.getProperty("user.dir")));
      LDAPSearcher.errTrace(System.getProperty("user.dir"), 3);
      InputStream isGrep = pGrep.getInputStream();
      InputStreamReader isr = new InputStreamReader(isGrep);
      BufferedReader bfr = new BufferedReader(isr);

      String strLine = null;
      while ( (strLine = bfr.readLine()) != null) {
        System.out.println("linje: "+strLine);
      }

      int iExitVal = pGrep.waitFor();
      //System.out.println("Exitvalue: " + iExitVal);
      if (iExitVal == 0) {
        return new Tagging(str, "department", 0, 0);
      }*/

    }
    catch (Throwable t) {
      // nothing to do - just return false on error
      t.printStackTrace();
    }
    return null;
  }

  /**
   * Search for the given array of words in the department list.
   *
   * Will succeed if all the words are found in the same line in the list.
   *
   * @param str  the word to search for
   * @return     if the word was found, a corresponding {@link Tagger.Tagging} object,
   *             if none found records, the function returns <B>null<(B>.
   */
  /*private Tagging isDepartment(String[] astr) {
	  System.out.println("isdepartmenttab");
    String strRegExp = strGrepComb(astr);
    try {
      String strGrep[] = new String[] {
          "grep", "-i", *//*"-c",*/ /*"-q",*/
   /*       strRegExp,
          "deps.txt"};

      String an[] = null;

      Process pGrep = Runtime.getRuntime().exec(strGrep, null,
                                                new File(System.getProperty(
          "user.dir")));
      LDAPSearcher.errTrace(System.getProperty("user.dir"), 3);
      InputStream isGrep = pGrep.getInputStream();
      InputStreamReader isr = new InputStreamReader(isGrep);
      BufferedReader bfr = new BufferedReader(isr);

      String strLine = null;
      while ( (strLine = bfr.readLine()) != null) {
        ;
      }

      int iExitVal = pGrep.waitFor();
      if (iExitVal == 0) {
        String strRetVal = astr[0];
        for (int i = 1; i < astr.length; i++) {
          strRetVal += "|" + astr[i];
        }
        System.out.println("returnerer ting fra grep..");
        return new Tagging(astr, "department", 0, 0);
      }

    }
    catch (Throwable t) {
      // nothing to do - just return false on error
    }
    return null;

  }*/

  private class Tagging
      implements Comparable {
    /**
     * The tagged string
     */
    public String m_strTagged;
    /**
     * The word concept the word is tagged as (class)
     *
     * Examples: firstname, lastname, department
     */
    public String m_strClass;
    /**
     * The key to use for the search
     *
     * Usually the same as the word.
     */
    public String m_strKey;
    /**
     * The number of times the word was found
     */
    public int m_iCount;
    /**
     * The number of times the word was found as a whole word.
     *
     * Makes it possible to separate the word Hans (as a first name)
     * from the word Hansen (which is a last name)
     */
    public int m_iExactCount;

    /**
     * Public constructor. Sets all the fields.
     *
     * @param  strTagged   the text that is tagged
     * @param  strClass    the class/concept the word is tagged as
     * @param  iCount      the number of times the word is found
     * @param  iCountExact the number of times the word is found as an exact match
     */
    public Tagging(String strTagged, String strClass, int iCount, int iCountExact) {
      m_strTagged = strTagged;
      m_strKey = m_strTagged;
      m_strClass = strClass;
      m_iCount = iCount;
      m_iExactCount = iCountExact;
    }

    /**
     * Public constructor. Sets all the fields.
     *
     * @param  astrTagged  the text that is tagged as an array. Will be concatenated with spaces between.
     * @param  strClass    the class/concept the word is tagged as
     * @param  iCount      the number of times the word is found
     * @param  iCountExact the number of times the word is found as an exact match
     */
    public Tagging(String astrTagged[], String strClass, int iCount, int iCountExact) {
		//System.out.println("strClass: "+strClass);
      m_iCount = iCount;
      m_strTagged = "";
      for (int i = 0; i < astrTagged.length; i++) {
        m_strTagged += astrTagged[i] + " ";
      }
      m_strTagged = m_strTagged.trim();
      m_strKey = m_strTagged;
      m_strClass = strClass;
      m_iExactCount = iCountExact;

    }

    /**
     * Replaces any ,  or  in beginning of word with corresponding ,  or .
     */
    public void capitalizeNorw()
    {
      m_strKey = m_strKey.replaceAll("\\b", "");
      m_strKey = m_strKey.replaceAll("\\b", "");
      m_strKey = m_strKey.replaceAll("\\b", "");
    }

    /**
     * Equal-comparison of two objects of this class
     *
     * Two Tagging objects are equal if all parameters are equal
     */
    public boolean equals(Object o) {
      if (! (o instanceof Tagging)) {
        return false;
      }
      Tagging t = (Tagging) o;
      return t.m_strTagged.equals(m_strTagged) &&
          t.m_strClass.equals(m_strClass) &&
          t.m_iCount == m_iCount;
    }

    /**
     * Creates a hashing code for the object
     */
    public int hashCode() {
      return 31 * 31 * m_iCount + 31 * m_strTagged.hashCode() +
          m_strClass.hashCode();
    }

    /**
     * Compare two objects of this class
     *
     * Tagging with higher exact count comes first, when equal the tagging with
     * the higher cound comes, and finaly the strings are compared.
     *
     * @param o    the object to compare to - should be a Tagging object
     * @return     a number showing the order of this two objects
     */
    public int compareTo(Object o) {
		//System.out.println("sammenligner: "+this+" med "+o);
      Tagging t = (Tagging) o;
      //System.out.println("Exactcounts: this: "+m_iExactCount+", other: "+t.m_iExactCount);
      //System.out.println("Other counts: this: "+m_iCount+", other: "+t.m_iCount);
      if (t.m_iExactCount != m_iExactCount && (t.m_iExactCount==0 || m_iExactCount==0)){
		  //System.out.println("forskjell i eksaktcount");
        return (t.m_iExactCount - m_iExactCount);
      }
      /*else if (t.m_iCount != m_iCount) {
        return (t.m_iCount - m_iCount);
      }*/
      /*else if (m_strTagged.equals(t.m_strTagged)) {
        return -m_strClass.compareTo(t.m_strClass);
      }*/
      //System.out.println("forskjell i streng");
      return t.m_strTagged.length()-m_strTagged.length(); //-m_strTagged.compareTo(t.m_strTagged);
    }

    /**
     * Creates a string of the object.
     *
     * The string is the XML representation that can be sendt to the client
     * application
     *
     * @return  A String object with the XML code representing this tagging
     */
    public String toString() {
      String strRet;
      strRet = XMLTagger.strTag("concept", m_strClass);
      strRet += XMLTagger.strTag("key", m_strKey);
      strRet += XMLTagger.strTag("data", m_strTagged);
      strRet = XMLTagger.strTag("apo", strRet);

      return strRet;
    }

    /**
     * Returns the first word in the m_strTagged field.
     *
     * Used to sort the tagges before they are returned.
     *
     * @return A String object containing the first word in the m_strTagged field
     */
    public String strWord() {
      int iPos = m_strTagged.indexOf(' ');
      if (iPos == -1) {
        return m_strTagged;
      }
      else {
        return m_strTagged.substring(0, iPos);
      }
    }

  }
}