import edu.wsu.KepheraSimulator.RobotController;
import edu.wsu.KepheraSimulator.KSGripperStates;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.lang.Math;
import java.io.*;


public class MarvinController extends RobotController{

	private Map locationDescriptions;

	private static int WEST = 1;
	private static int NORTH = 2;
	private static int EAST = 3;
	private static int SOUTH = 4;
	private static int LEFT = 1;
	private static int RIGHT = 2;

	private static double ADJUSTMENT_REFINEMENT = 0.19;
	private static int DIFF_TOLERANCE = 150;

	private boolean wall_left;
	private boolean wall_right;
	private boolean wall_front;
	private boolean wall_back;

	private boolean turning;				//True if Marvin is in a turning mode
	private boolean moving;				//True if Marvin is in a moving mode
	private boolean verifying;
	private boolean positioning;

	private boolean forceXMovement;
	private boolean forceYMovement;

	private int xDistanceToGo;
	private int yDistanceToGo;
	private int direction;
	private int currentlyTurningTo;

	private ArrayList commandList;

	protected long theTurnDegree = 90;
	protected int state = 0;
	private int verifyState = 0;
	protected boolean slowSpeed = false; // check if we're compensating.
	protected long degreeTarget = 0;
	protected long toMove = 0;
	long startCount = 0;
	private int targetDoorLocation = 0;
	private int targetRoomNumber = 0;
	private int[] targetWalls;
	private int currentRoom = 25;
	private int[] adjustments;

	public MarvinController(){

		initMapDescriptions();
		targetWalls = new int[4];
		adjustments = new int[2];
		commandList = new ArrayList();
		xDistanceToGo = 0;
		yDistanceToGo = 0;
		direction = EAST;
		turning = false;
		moving = false;
		forceYMovement = false;
		forceXMovement = false;
		state = 0;
		setWaitTime(5);

		getLocationFromFile();
		setRobotReady();
	}


	public void doWork() throws Exception {


		//Marvin isn't in turning or moving mode, but there is some distance to go in either x or y direction
		if(!(xDistanceToGo==0 && yDistanceToGo==0) && !verifying && !turning && !moving && !positioning){

			//North
			if(yDistanceToGo < 0 && !forceXMovement){
				if(direction != NORTH){
					turn(NORTH);
				}
				else{
					moving=true;
				}
			}
			//South
			else if(yDistanceToGo > 0 && !forceXMovement){
				if(direction != SOUTH){
					turn(SOUTH);
				}
				else{
					moving=true;
				}
			}
			//Left
			else if(xDistanceToGo < 0 && !forceYMovement){
				if(direction != WEST){
					turn(WEST);
				}
				else{
					moving=true;
				}
			}
			//Right
			else if(xDistanceToGo > 0 && !forceYMovement){
				if(direction != EAST){
					turn(EAST);
				}
				else{
					moving=true;
				}
			}
		}

		//Marvin is in turning mode
		else if(turning){
			if(state == 0) {
				degreeTarget = getLeftWheelPosition() + theTurnDegree*3;
				if(theTurnDegree > 0) currentlyTurningTo = RIGHT;			//NEW
				else if(theTurnDegree < 0) currentlyTurningTo = LEFT;		//NEW
				slowSpeed = false;
				state = 1;
			}

			if(state == 1) {

				// Sjekker om roboten har snudd langt nok.
				if(getLeftWheelPosition() == degreeTarget) {
					setMotorSpeeds(0,0);
					state = 2; // Good! Next step
				}
				else {

					// Checks if the robot has overturned and must adjust
					if (((getLeftWheelPosition() < degreeTarget) && (theTurnDegree < 0)) ||
					   (getLeftWheelPosition() > degreeTarget) && (theTurnDegree > 0)) {

					//if ( ((getLeftWheelPosition() < degreeTarget) && currentlyTurningTo == LEFT) ||			//NEW
					//		((getLeftWheelPosition() > degreeTarget) && currentlyTurningTo == RIGHT) ){		//NEW

						long offset = (getLeftWheelPosition() - degreeTarget);

						theTurnDegree = -offset*3;
						degreeTarget = getLeftWheelPosition() - offset;
						slowSpeed = true;
					}

					int speed;
					if(slowSpeed)
						speed = 1;
					else
						speed = 2;

					// keep on turning
					if(theTurnDegree > 0) {
						setMotorSpeeds(speed, -speed); // Right turn
					}
					else {
						setMotorSpeeds(-speed, speed); // Left turn
					}

				}
			}

			if(state == 2) {
			  turning = false;
			 // slowSpeed = false;
			  if(currentlyTurningTo == RIGHT) changeDirectionRight();
			  else if(currentlyTurningTo == LEFT) changeDirectionLeft();
			  theTurnDegree = 0;
			  state = 0;
			}
		}

		//Marvin is in moving mode
		else if(moving){
			if(state == 0) {
				startCount = getLeftWheelPosition();
				if(direction==EAST || direction==WEST) toMove = Math.abs(xDistanceToGo);
				else if(direction== NORTH || direction==SOUTH) toMove = Math.abs(yDistanceToGo);
				else System.out.println("Error trying to move (no direction)");
			    state = 1;
			}

			if(state == 1) {
				int[] readings = getAvgReadings();

				//What to do if there is a wall ahead
				if(readings[1] > 1000 || readings[2] > 1000 || readings[3] > 1000 || readings[4] > 1000){
					setMotorSpeeds(0,0);

					//If the current command involves movement in another direction
					if( (direction==NORTH || direction==SOUTH) && xDistanceToGo != 0){
						state = 2;
						forceXMovement = true;
						forceYMovement = false;
					}
					else if( (direction==EAST || direction==WEST) && yDistanceToGo != 0){
						state = 2;
						forceYMovement = true;
						forceXMovement = false;
					}

					//If the current command does not involve movement in another direction, try to avoid the wall
					else if(readings[1] > readings[4] && direction==NORTH){ xDistanceToGo += 40; adjustments[0] += 40;}
					else if(readings[4] > readings[1] && direction==NORTH){ xDistanceToGo -= 40; adjustments[0] -= 40;}
					else if(readings[1] > readings[4] && direction==SOUTH){ xDistanceToGo -= 40; adjustments[0] -= 40;}
					else if(readings[4] > readings[1] && direction==SOUTH){ xDistanceToGo += 40; adjustments[0] += 40;}
					else if(readings[1] > readings[4] && direction==WEST){ yDistanceToGo -= 40; adjustments[1] -= 40;}
					else if(readings[4] > readings[1] && direction==WEST){ yDistanceToGo += 40; adjustments[1] += 40;}
					else if(readings[1] > readings[4] && direction==EAST){ yDistanceToGo += 40; adjustments[1] += 40;}
					else if(readings[4] > readings[1] && direction==EAST){ yDistanceToGo -= 40; adjustments[1] -= 40;}


				}

				//What to do if there is more distance to go
				else if((getLeftWheelPosition()-startCount) < toMove) {

			    	//If there's a very short distance to go, go slowly!!
			    	if(Math.abs( (getLeftWheelPosition()-startCount)-toMove ) < 10){
						setMotorSpeeds(1,1);
					}
					else setMotorSpeeds(5,5);
			    }

			    //If there is no distance to go and no wall ahead, update distance to go and travel
			    else if((getLeftWheelPosition()-startCount) >= toMove) {
			        setMotorSpeeds(0,0);
			        forceXMovement = false;
			    	forceYMovement = false;
			        state = 2;
			    }
			}

			if(state == 2) {
				setMotorSpeeds(0,0);
				if(direction==WEST) xDistanceToGo += (getLeftWheelPosition()-startCount);
				else if(direction==EAST) xDistanceToGo -= (getLeftWheelPosition()-startCount);
				else if(direction==NORTH) yDistanceToGo += (getLeftWheelPosition()-startCount);
				else if(direction==SOUTH) yDistanceToGo -= (getLeftWheelPosition()-startCount);
			    updateTravel((int)(getLeftWheelPosition()-startCount));
				moving = false;
				state = 0;
System.out.println("Move state finished, xDistanceToGo="+xDistanceToGo+", yDistanceToGo="+yDistanceToGo);
      		}
		}

		//Marvin is in verify location mode
		else if(verifying){
			if(verifyState == 1){
				turn(targetDoorLocation);
				verifyState = 2;
			}
			else if(verifyState == 2){
				findWalls();
				if(wallsAreCorrect()){
					System.out.println("Verified location: Room "+(300+targetRoomNumber));
					if(currentRoom != targetRoomNumber) setMsg("Verified location: "+(String)locationDescriptions.get(new Integer(targetRoomNumber)));
					currentRoom = targetRoomNumber;
					setMarvinLocation(currentRoom);

					//Start positioning at the door
					positioning = true;
				}
				else{

					System.out.println("Couldn't verify location, assuming to be near the goal "+targetRoomNumber+". Searching for the door.");
					setMsg("Couldn't verify location, assuming to be near "+(String)locationDescriptions.get(new Integer(targetRoomNumber))+". Searching for the location...");
					currentRoom = targetRoomNumber;
					setMarvinLocation(currentRoom);

					//Start positioning, attempting to find the door
					positioning = true;


					/*
					System.out.println("Couldn't verify location. The robot doesn't know where it is and will stop.");
					while(!commandList.isEmpty()){
						commandList.remove(0);
					}
					*/
				}
				targetRoomNumber = 0;
				targetDoorLocation = 0;
				verifyState = 0;
				verifying = false;
			}
		}

		//Marvin is in "position yourself at the location" mode
		else if(positioning){
			int[] readings = getAvgReadings();

//Just for debugging
for(int i=0; i<8; i++){
System.out.print(readings[i]+" ");
}
System.out.println("Direction: "+direction+"\n");

			//If there is a wall on either side, reduce corner sensor's readings for that side
			//(because it will read more than if there was no wall there, even if the robot is in the middle of the door opening
			if(readings[0] > 800 && readings[1] > 200) readings[1] -= 200;
			if(readings[5] > 800 && readings[4] > 200) readings[4] -= 200;

			//If the difference between corner sensors is too big, adjust position
			if(Math.abs(readings[1]-readings[4]) > DIFF_TOLERANCE){
				if(readings[1] > readings[4]) addAdjustCommand(1, Math.abs(readings[1]-readings[4]));  //Right
				else if(readings[4] > readings[1]) addAdjustCommand(0, Math.abs(readings[1]-readings[4])); //Left
			}

			//If the difference between corner sensors are ok, but the values are to small, go a bit forward (towards the door hopefully)
			else if(Math.abs(readings[1]-readings[4]) < DIFF_TOLERANCE && (readings[1]+readings[4] < DIFF_TOLERANCE)){
				addAdjustCommand(2, 40); //Forward
			}
			positioning = false;
		}

		//Marvin has no distance to go and isn't moving or turning. Verifying that the location is correct
		else if(xDistanceToGo == 0 && yDistanceToGo == 0 && targetRoomNumber != 0 && !turning && !moving && !verifying && !positioning){

			setMotorSpeeds(0,0);
			verifyState = 1;
			verifying = true;
		}

		//Get next command or read a command list from file
		else{

			setMotorSpeeds(0,0);
			if(commandList.isEmpty()){

				//Wait for 1 second, so that we aren't reading the path file all the time
				sleep(1000);
				String result ="";
				try{
					//Read to buffer
					FileReader fr = new FileReader("./controllers/path");
					File infile = new File("./controllers/path");
					char[] cbuf = new char[(int)infile.length()];
					fr.read(cbuf);
					fr.close();

					//Buffer to string
					for(int i=0; i<cbuf.length; i++){
						result += cbuf[i];
					}

					//Clear path file
					FileWriter fw = new FileWriter("./controllers/path");
					fw.write(".");
					fw.close();
				}
				catch(Exception e){
					System.out.println("Error dealing with path file in MarvinController");
					System.out.println(e.toString());
				}
				parseCommandString(result);
			}
			else if(!commandList.isEmpty()){
				getNextCommand();
			}
		}
	}

	public void executeCommands(ArrayList commands){
		commandList = commands;
	}


	public void close() throws Exception {



	}

//-----------------------------Private methods-------------------------------------------

	private void addAdjustCommand(int dir, int diff){  //2 for forward, 1 for right, 0 for left
		getLocationFromFile();
		int[] adjustment = new int[2];
		adjustment[0] = 0; adjustment[1] = 0;

		if(dir == 0){
			if(direction == WEST) adjustment[1] = (int)(ADJUSTMENT_REFINEMENT*diff);
			else if(direction == NORTH) adjustment[0] = -(int)(ADJUSTMENT_REFINEMENT*diff);
			else if(direction == EAST) adjustment[1] = -(int)(ADJUSTMENT_REFINEMENT*diff);
			else if(direction == SOUTH) adjustment[0] = (int)(ADJUSTMENT_REFINEMENT*diff);
		}
		else if(dir == 1){
			if(direction == WEST) adjustment[1] = -(int)(ADJUSTMENT_REFINEMENT*diff);
			else if(direction == NORTH) adjustment[0] = (int)(ADJUSTMENT_REFINEMENT*diff);
			else if(direction == EAST) adjustment[1] = (int)(ADJUSTMENT_REFINEMENT*diff);
			else if(direction == SOUTH) adjustment[0] = -(int)(ADJUSTMENT_REFINEMENT*diff);
		}
		else if(dir == 2){
			if(direction == WEST) adjustment[0] = -diff;
			else if(direction == NORTH) adjustment[1] = -diff;
			else if(direction == EAST) adjustment[0] = diff;
			else if(direction == SOUTH) adjustment[1] = diff;
		}

		int[] blah = new int[9];
		blah[0] = adjustment[0];
		blah[1] = adjustment[1];
		blah[2] = 0;
		blah[3] = currentRoom;
		blah[4] = currentRoom;

		for(int i=0;i<4;i++){
			blah[5+i] = targetWalls[i];
		}

		commandList.add(commandList.size(),blah);
		blah = new int[9];
	}

	private void parseCommandString(String s){
		if(s.equals(".")) System.out.println("No new commands");
		else{
			StreamTokenizer st = new StreamTokenizer(new StringReader(s));
			st.ordinaryChar('{');
			st.ordinaryChar('}');
			st.ordinaryChar(',');
			st.ordinaryChar('.');
			try{
				int tokenType = st.nextToken();
				int[] blah = new int[9];
				while(tokenType != StreamTokenizer.TT_EOF){
					if(st.ttype == StreamTokenizer.TT_WORD && st.sval.equals("From")){
						st.nextToken();
						st.nextToken();
						blah[3]=(int)st.nval;
					}
					else if(st.ttype == StreamTokenizer.TT_WORD && st.sval.equals("To")){
						st.nextToken();
						st.nextToken();
						blah[4]=(int)st.nval;
					}
					else if(st.ttype=='{'){
						st.nextToken();
						blah[0] = (int)st.nval;
						st.nextToken();
						st.nextToken();
						blah[1] = (int)st.nval;
						st.nextToken();
						st.nextToken();
						blah[5] = (int)st.nval;
						st.nextToken();
						st.nextToken();
						blah[6] = (int)st.nval;
						st.nextToken();
						st.nextToken();
						blah[7] = (int)st.nval;
						st.nextToken();
						st.nextToken();
						blah[8] = (int)st.nval;

						commandList.add(0, blah);
						blah = new int[9];
					}
					tokenType = st.nextToken();
				}
			}
			catch(Exception e){
				System.out.println("Error parsing path file in MarvinController");
				e.printStackTrace(System.out);
			}

			if(!commandList.isEmpty()){
				int[] tempStart = (int[])commandList.get(commandList.size()-1);
				int[] tempGoal = (int[])commandList.get(0);
				setMsg("New task: Go from "+(String)locationDescriptions.get(new Integer(tempStart[3]))+" to "+(String)locationDescriptions.get(new Integer(tempGoal[4]))+".");
			}
		}
	}

	private void getNextCommand(){
		int[] blah = (int[])commandList.remove(commandList.size()-1);
		getLocationFromFile();

		if(blah[3] == currentRoom){

			//Setting the X and Y distances to go that are given in the command
			//Also trying to compensate for previous adjustments that was made due to blocking objects
			xDistanceToGo = blah[0] + adjustments[0];
			adjustments[0] = 0;
			yDistanceToGo = blah[1] + adjustments[1];
			adjustments[1] = 0;
			//targetDoorLocation = blah[2];
			targetRoomNumber = blah[4];

			System.out.println("New command! Distances to go: "+xDistanceToGo+", "+yDistanceToGo+"\n"+
									"Going from room "+blah[3]+" to "+targetRoomNumber+". Currently at room "+currentRoom);
			//setMsg("New command!"+(String)locationDescriptions.get(new Integer(25)));
			for(int i=0;i<4;i++){
				targetWalls[i] = blah[5+i];
				if(targetWalls[i]==2){
					if(i==0) targetDoorLocation = WEST;
					else if(i==1) targetDoorLocation = NORTH;
					else if(i==2) targetDoorLocation = EAST;
					else if(i==3) targetDoorLocation = SOUTH;
				}
			}
		}
		else{
			System.out.println("New command found, but start location ("+blah[3]+") didn't match the robot's location "+(currentRoom)+". Command cancelled.");
		}
	}

	private void findWalls(){

		int[] getDistanceValue = getAvgReadings();

		//Assume there is a wall on the left if the left sensor gives a reading bigger than 300
		if(getDistanceValue[0] > 250) wall_left = true;
		else wall_left = false;

		//Assume there is a wall in front, if the front sensors have a combined reading of more than 600, and each have a reading of more than 300
		if( (getDistanceValue[2] + getDistanceValue[3] > 500)
			&&
			(getDistanceValue[2] > 250)
			&&
			(getDistanceValue[3] > 250)
			){
				wall_front = true;
		}
		else wall_front = false;

		//Assume there is a wall on the right if the right sensor gives a reading bigger than 300
		if(getDistanceValue[5] > 250) wall_right = true;
		else wall_right = false;

		if(getDistanceValue[6] + getDistanceValue[7] > 600) wall_back = true;
		else wall_back = false;
	}

	private boolean wallsAreCorrect(){

		boolean expectedLeft = false;
		boolean expectedRight = false;
		boolean expectedBack = false;

		if(direction==WEST){
			if(targetWalls[3]==1) expectedLeft = true;
			if(targetWalls[1]==1) expectedRight = true;
			if(targetWalls[2]==1) expectedBack = true;
		}
		else if(direction==NORTH){
			if(targetWalls[0]==1) expectedLeft = true;
			if(targetWalls[2]==1) expectedRight = true;
			if(targetWalls[3]==1) expectedBack = true;
		}
		else if(direction==EAST){
			if(targetWalls[1]==1) expectedLeft = true;
			if(targetWalls[3]==1) expectedRight = true;
			if(targetWalls[0]==1) expectedBack = true;
		}
		else if(direction==SOUTH){
			if(targetWalls[2]==1) expectedLeft = true;
			if(targetWalls[0]==1) expectedRight = true;
			if(targetWalls[1]==1) expectedBack = true;
		}

		if(
			(wall_left==expectedLeft)
			&&
			(wall_right==expectedRight)
			&&
			(wall_back==expectedBack)
			&&
			(!wall_front)

			)
			 {

			return true;
		}
		else{
			return false;
		}
	}

	private int[] getAvgReadings(){
		int[] returnValues = new int[8];
		for(int i=0;i<8;i++){
			int summer=0;
			for(int j=0;j<10;j++){
				summer += getDistanceValue(i);
			}
			returnValues[i] = summer/10;
		}
		return returnValues;
	}

	private void turn(int a){
		if(a == direction){
			turning=false; theTurnDegree = 0;
		}
		else{
			if(a == WEST && direction == SOUTH) theTurnDegree = 90;
			else if(a == WEST) theTurnDegree = -90;
			else if(a == NORTH && direction == WEST) theTurnDegree = 90;
			else if(a == NORTH) theTurnDegree = -90;
			else if(a == EAST && direction == NORTH) theTurnDegree = 90;
			else if(a == EAST) theTurnDegree = -90;
			else if(a == SOUTH && direction == EAST) theTurnDegree = 90;
			else if(a == SOUTH) theTurnDegree = -90;

	//		if(a != direction) turning = true;
			turning = true;
		}
	}

	private void changeDirectionRight(){
		if(direction==WEST) direction=NORTH;
		else if(direction==NORTH) direction=EAST;
		else if(direction==EAST) direction=SOUTH;
		else if(direction==SOUTH) direction=WEST;
	}

	private void changeDirectionLeft(){
			if(direction==WEST) direction=SOUTH;
			else if(direction==NORTH) direction=WEST;
			else if(direction==EAST) direction=NORTH;
			else if(direction==SOUTH) direction=EAST;
	}

	private void updateTravel(int t){
		try{

			String marvinFile = "";
			FileReader fr = new FileReader("./controllers/marvin");
			File infile = new File("./controllers/marvin");
			char[] cbuf = new char[(int)infile.length()];
			fr.read(cbuf);
			fr.close();
			//Buffer to string
			for(int i=0; i<cbuf.length; i++){
				marvinFile += cbuf[i];
			}
			StringBuffer sb = new StringBuffer(marvinFile);
			int writePosition = 0;

			for(int i=0; i<sb.length(); i++){
				if(i+17 > sb.length()){
					System.out.println("Didn't find 'DistanceTravelled' in file marvin");
					i=sb.length();
				}
				else if(sb.substring(i, i+17).equals("DistanceTravelled")){
					writePosition = i+18;
					i = sb.length();
				}
			}

			if(writePosition != 0){
				int readPos = writePosition;
				StringBuffer oldNumber = new StringBuffer();
				while(Character.isDigit(sb.charAt(readPos))){
					oldNumber.append(sb.charAt(readPos));
					readPos++;
				}
				String oldNumberString = oldNumber.toString();
				int oldNumberInt = Integer.parseInt(oldNumberString);
				int newNumber = oldNumberInt+t;
				String newNumberString = ""+newNumber+"\n";
				sb = sb.replace(writePosition, writePosition+newNumberString.length(), newNumberString);
				FileWriter fw = new FileWriter("./controllers/marvin");
				fw.write(sb.toString());
				fw.close();
			}



		}
		catch(Exception e){
			System.out.println("Error writing to file marvin");
			e.printStackTrace(System.out);
		}
	}

	private void setRobotReady(){
		try{
			String toWrite ="yes";

			String marvinFile = "";
			FileReader fr = new FileReader("./controllers/marvin");
			File infile = new File("./controllers/marvin");
			char[] cbuf = new char[(int)infile.length()];
			fr.read(cbuf);
			fr.close();
			//Buffer to string
			for(int i=0; i<cbuf.length; i++){
				marvinFile += cbuf[i];
			}
			StringBuffer sb = new StringBuffer(marvinFile);
			int writePosition = 0;

			for(int i=0; i<sb.length(); i++){
				if(i+8 > sb.length()){
					System.out.println("Didn't find 'SimulatorReady' in file marvin");
					i=sb.length();
				}
				else if(sb.substring(i, i+14).equals("SimulatorReady")){
					writePosition = i+15;
					i = sb.length();
				}
			}

			if(writePosition != 0){
				sb = sb.replace(writePosition, writePosition+3, toWrite);
				FileWriter fw = new FileWriter("./controllers/marvin");
				fw.write(sb.toString());
				fw.close();
			}
		}
		catch(Exception e){
			System.out.println("Error writing to file marvin");
			e.printStackTrace(System.out);
		}
	}

	private void setMarvinLocation(int location){
			try{
				String toWrite ="";
				if(location < 10) toWrite = "  "+location;
				else if(location <100) toWrite = " "+location;
				else toWrite = ""+location;

				String marvinFile = "";
				FileReader fr = new FileReader("./controllers/marvin");
				File infile = new File("./controllers/marvin");
				char[] cbuf = new char[(int)infile.length()];
				fr.read(cbuf);
				fr.close();
				//Buffer to string
				for(int i=0; i<cbuf.length; i++){
					marvinFile += cbuf[i];
				}
				StringBuffer sb = new StringBuffer(marvinFile);
				int writePosition = 0;

				for(int i=0; i<sb.length(); i++){
					if(i+8 > sb.length()){
						System.out.println("Didn't find 'Location' in file marvin");
						i=sb.length();
					}
					else if(sb.substring(i, i+8).equals("Location")){
						writePosition = i+10;
						i = sb.length();
					}
				}

				if(writePosition != 0){
					sb = sb.replace(writePosition, writePosition+3, toWrite);
					FileWriter fw = new FileWriter("./controllers/marvin");
					fw.write(sb.toString());
					fw.close();
				}



			}
			catch(Exception e){
				System.out.println("Error writing to file marvin");
				e.printStackTrace(System.out);
			}

	}

	private void getLocationFromFile(){
		int locat = 0;
				try{
					FileReader fr = new FileReader("./controllers/marvin");
					File infile = new File("./controllers/marvin");
					char[] cbuf = new char[(int)infile.length()];
					fr.read(cbuf);
					fr.close();

					String marvinFile = "";
					for(int i=0; i<cbuf.length; i++){
						marvinFile += cbuf[i];
					}

					StreamTokenizer st = new StreamTokenizer(new StringReader(marvinFile));
					st.lowerCaseMode(true);
					int tokenType = st.nextToken();
					while(st.ttype != StreamTokenizer.TT_EOF && (st.sval == null || !((st.sval).equals("location")))){
							st.nextToken();
					}
					if(tokenType == StreamTokenizer.TT_EOF) locat = 0;
					else if(tokenType == StreamTokenizer.TT_WORD && st.sval.equals("location")){
						st.nextToken();
						tokenType = st.nextToken();
						if (tokenType == StreamTokenizer.TT_NUMBER) locat = (int)st.nval;
					}
					else locat = 0;
				}
				catch(Exception e){
					System.out.println("Error trying to read Marvin's location from file marvin");
					e.printStackTrace(System.out);
				}

		currentRoom = locat;
	}

	private void initMapDescriptions(){
		locationDescriptions = new HashMap();
		locationDescriptions.put(new Integer(25), "Entrance");
		locationDescriptions.put(new Integer(22), "Door 322");
		locationDescriptions.put(new Integer(21), "Door 321");
		locationDescriptions.put(new Integer(20), "Door 320");
		locationDescriptions.put(new Integer(63), "Door 363");
		locationDescriptions.put(new Integer(26), "Door 326");
		locationDescriptions.put(new Integer(24), "Door 324");
		locationDescriptions.put(new Integer(17), "Door 317");
		locationDescriptions.put(new Integer(16), "Door 316");
		locationDescriptions.put(new Integer(15), "Door 315");
		locationDescriptions.put(new Integer(14), "Door 314");
		locationDescriptions.put(new Integer(13), "Door 313");
		locationDescriptions.put(new Integer(12), "Door 312");
		locationDescriptions.put(new Integer(11), "Door 311");
		locationDescriptions.put(new Integer(10), "Door 310");
		locationDescriptions.put(new Integer(8), "Door 308");
		locationDescriptions.put(new Integer(7), "Door 307");
		locationDescriptions.put(new Integer(6), "Door 306");
		locationDescriptions.put(new Integer(5), "Door 305");
		locationDescriptions.put(new Integer(4), "Door 304");
		locationDescriptions.put(new Integer(3), "Door 303");
		locationDescriptions.put(new Integer(2), "Door 302");
		locationDescriptions.put(new Integer(1), "Door 301");
		locationDescriptions.put(new Integer(42), "Door 342");
		locationDescriptions.put(new Integer(431), "Door 343b");
		locationDescriptions.put(new Integer(43), "Door 343");
		locationDescriptions.put(new Integer(44), "Door 344");
		locationDescriptions.put(new Integer(461), "Door 346b");
		locationDescriptions.put(new Integer(45), "Door 345");
		locationDescriptions.put(new Integer(46), "Door 346");
		locationDescriptions.put(new Integer(47), "Door 347");
		locationDescriptions.put(new Integer(90), "Elevator");
		locationDescriptions.put(new Integer(30), "Door 330 (stairs)");
		locationDescriptions.put(new Integer(49), "Door 349");
		locationDescriptions.put(new Integer(48), "Door 348");
		locationDescriptions.put(new Integer(31), "Door 331");
		locationDescriptions.put(new Integer(32), "Door 332");
		locationDescriptions.put(new Integer(33), "Door 333");
		locationDescriptions.put(new Integer(34), "Door 334");
		locationDescriptions.put(new Integer(35), "Door 335");
		locationDescriptions.put(new Integer(36), "Door 336");
		locationDescriptions.put(new Integer(37), "Door 337");
		locationDescriptions.put(new Integer(54), "Door 354");
		locationDescriptions.put(new Integer(56), "Door 356");
		locationDescriptions.put(new Integer(58), "Door 358");
		locationDescriptions.put(new Integer(601), "Door 360b");
		locationDescriptions.put(new Integer(59), "Door 359");
		locationDescriptions.put(new Integer(60), "Door 360");
		locationDescriptions.put(new Integer(61), "Door 361");
		locationDescriptions.put(new Integer(63), "Door 363");
		locationDescriptions.put(new Integer(50), "Toilets");
	}
}